/*
(c) 2005, Supratik Mukhopadhayay, Nayana Majumdar
*/
#ifndef _Isles_H_
#define _Isles_H_

#ifdef DEFINE_ISLESGLOBAL
#define ISLESGLOBAL
#else
#define ISLESGLOBAL extern
#endif

#include <math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>

#include <Vector.h>

// Normally these values are kept at 7, 14 and 18
#define MINDIST 1.0e-8	// Changes on 09Oct07 for Jackson; on 15/12/2011 for MHSP even holes
#define MINDIST2 1.0e-16	// Changes on 09Oct07 for Jackson; on 15/12/2011 for MHSP even holes
#define MINDIST3 1.0e-20	// Changes on 09Oct07 for Jackson; on 15/12/2011 for MHSP even holes
// #define MINDIST 1.0e-8	// Change necessary for Jackson wedge problem
// #define MINDIST2 1.0e-16	// Changes necessary for Jackson wedge problem

#define ARMAX 10000.0	// Maximum aspect ratio for an element

#define FarField 10.0	// Beyond FarField*ElementSize, use nodal expressions

// #define XNSegApprox 10	// much less time but inaccurate results
// #define ZNSegApprox 10	// much less time but inaccurate results
#define XNSegApprox 100	// barely acceptable results
#define ZNSegApprox 100	// barely acceptable result
// #define XNSegApprox 1000	// much better results but lot more time
// #define ZNSegApprox 1000	// much better results but lot more time

// #define ST_PI 3.14159265
#define ST_PI 3.14159265358979323846

ISLESGLOBAL char ISLESVersion[10];

// Nb of times these get called: FailureCntr keeps a tab of how many times
// the exact expressions failed to evaluate.
ISLESGLOBAL int IslesCntr, ExactCntr, ApproxCntr, FailureCntr;
ISLESGLOBAL int ApproxFlag, DebugISLES;
ISLESGLOBAL FILE *fIsles;

ISLESGLOBAL int ExactRecSurf(double X, double Y, double Z,
																	double xlo, double zlo,
            											double xhi, double zhi,
																	double *Potential, Vector3D *Flux);

ISLESGLOBAL int ApproxRecSurf(double X, double Y, double Z,
																			double xlo, double zlo,
            													double xhi, double zhi,
																			int xseg, int zseg,
																			double *Potential, Vector3D *Flux);

ISLESGLOBAL int ExactTriSurf(double zMax, double X, double Y, double Z,
																	double *Potential, Vector3D *Flux);

ISLESGLOBAL int ApproxTriSurf(double zMax, double X, double Y, double Z,
																	int xseg, int zseg,
																	double *Potential, Vector3D *Flux);

ISLESGLOBAL int ExactRecVol(double a, double b, double c,
																		double X, double Y, double Z,
																		double *Potential, Vector3D *Flux);

ISLESGLOBAL double ExactCentroidalP_W(double rW, double lW);

ISLESGLOBAL double ExactAxialP_W(double rW, double lW, double Z);

ISLESGLOBAL double ExactAxialFZ_W(double rW, double lW, double Z);

ISLESGLOBAL double ApproxP_W(double rW, double lW, double X, double Y, double Z, int zseg);

ISLESGLOBAL double ApproxFX_W(double rW, double lW,
					double X, double Y, double Z, int zseg);

ISLESGLOBAL double ApproxFY_W(double rW, double lW,
					double X, double Y, double Z, int zseg);

ISLESGLOBAL double ApproxFZ_W(double rW, double lW,
					double X, double Y, double Z, int zseg);

ISLESGLOBAL int ApproxWire(double rW, double lW, double X, double Y, double Z,
														int zseg, double *potential, Vector3D *Flux);

ISLESGLOBAL double ImprovedP_W(double rW, double lW, double X, double Y, double Z);

ISLESGLOBAL double ImprovedFX_W(double rW, double lW, double X, double Y, double Z);

ISLESGLOBAL double ImprovedFY_W(double rW, double lW, double X, double Y, double Z);

ISLESGLOBAL double ImprovedFZ_W(double rW, double lW, double X, double Y, double Z);

ISLESGLOBAL int ImprovedWire(double rW, double lW, double X, double Y, double Z,
															double *potential, Vector3D *Flux);

ISLESGLOBAL double ExactThinP_W(double rW, double lW, double X, double Y, double Z);

ISLESGLOBAL double ExactThinFX_W(double rW, double lW, double X, double Y, double Z);

ISLESGLOBAL double ExactThinFY_W(double rW, double lW, double X, double Y, double Z);

ISLESGLOBAL double ExactThinFZ_W(double rW, double lW, double X, double Y, double Z);

ISLESGLOBAL int ExactThinWire(double rW, double lW, double X, double Y,
															double Z, double *potential, Vector3D *Flux);

// All coordiantes and vectors are in the global co-ordinate system.
// In case of area (both triangular and rectangular), it is assumed that
// the +ve normal direction is obtained by
// \vec{P1-P0} \cross \vec{P2-P1}.
// Note that, according to our convention, the second point of a right
// triangle \vec{P1} is the right corner of the triangle.
ISLESGLOBAL double ExactPointP(Point3D *SourcePt, Point3D *FieldPt);

ISLESGLOBAL int ExactPointF(Point3D *SourcePt, Point3D *FieldPt,
														Vector3D *Flux);

ISLESGLOBAL double ExactLineP(Point3D *SourcePt, Point3D FieldPt);	// line has two points

ISLESGLOBAL int ExactLineF(Point3D *SourcePt, Point3D FieldPt, Vector3D *Flux);

ISLESGLOBAL double ExactAreaP(int NbVertices, Point3D *SourcePt, Point3D FieldPt);

ISLESGLOBAL int ExactAreaF(int NbVertices, Point3D *SourcePt, Point3D FieldPt, Vector3D *Flux);

ISLESGLOBAL double ExactVolumeP(int NbPts, Point3D *SourcePt, Point3D FieldPt);

ISLESGLOBAL int ExactVolumeF(int NbPts, Point3D *SourcePt, Point3D FieldPt, Vector3D *Flux);

ISLESGLOBAL int Sign(double x);

#endif
