// This code can work as a template for codes calling neBEM toolkit
// The user has to supply
// int main(void)

// In addition, the user has to supply an interface between the code and
// neBEM which contains the following and usually resides in the
// src/Interface directory. Few templates exist there for easy reference.
// int neBEMGetNbPrimitives(void)
// int neBEMGetPrimitive(int prim, int *nvertex,
//                       double xvert[], double yvert[], double zvert[],
//                       double *xnorm, double *ynorm, double *znorm,
//                       int *volref1, int *volref2)
// int neBEMVolumeDescription(int volref1, int *shape1,
//                            int *material1, double *epsilon1,
//                            double *potential1, double *charge1,
//                            int *boundarytype1)
// int neBEMVolumePoint(double x, double y, double z)
// int neBEMGetPeriodicities(int *ix, double *sx, int *iy, double *sy,
//                             int *iz, double *sz)
// The Makefile has to be modified appropriately to incorporate these two
// source codes while building the executable.

// Analyze a triple GEM

#include <stdio.h>
#include <assert.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>


int main(void)
{
int PostProcess(void);	// a typical post-processing function

// beginning of neBEM computation - assign the state variable
neBEMState = 0;
int fstatus = neBEMInitialize();
assert(fstatus == 0);

// The following function uses neBEMGetNbPrimitives, neBEMGetPrimitive,
// neBEMVolumeDescription and neBEMVolumePoint, the last one used sparingly
fstatus = neBEMReadGeometry();
assert(fstatus == 0);

// It is likely that a routine will be supplied either by neBEM or the user
// to analyze the pritimives and to decide the size and shape of elements
// necessary on each primitive
// There is already a routine named AnalyzePrimitiveShape in ReTrim.c.
// A similar routine to determine the primitive characteristics and following
// discretization requirements can be implemented within the neBEMDiscretize
// function.
// This can be made more intelligent by making the larger integer take the
// longer side of the primitive
int **elementNbs;
elementNbs = imatrix(1, NbPrimitives, 1, 2);
for(register int prim = 1; prim <= NbPrimitives; ++prim)
	{
	elementNbs[prim][1] = tmpNbXSegs[prim];
	elementNbs[prim][2] = tmpNbZSegs[prim];
	}
fstatus = neBEMDiscretize(elementNbs);
assert(fstatus == 0);

// Since a boundary condition is likely to be
// associated with each element, this routine can be called only after the
// primitives have been discretized
fstatus = neBEMBoundaryConditions();
assert (fstatus == 0);

// Finally, request the solution. Make sure that the necessary flags and
// counters are properly set. If not, set them here.
// Please note that the flags maintain a hierarchy. For example, it NewDevice
// is set to 1, all the rest flags are simply ignored and a fresh calculation
// is initiated.
TimeStep = 1; EndOfTime = 1;
fstatus = neBEMSolve();	// fresh calculation
assert(fstatus == 0);

// Post-processing can be carried out with separate function(s) written within
// this code, or function(s) residing in a separate source code, or a mix.
fstatus = PostProcess();
assert(fstatus == 0);

// End neBEM gracefully
fstatus = neBEMEnd();
assert(fstatus == 0);

return 0;
}	// main ends
