// Conducting plate example

#include <stdio.h>
#include <assert.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>


int main(void)
{
int SpecifyKnownCharges(void);	// function to assign known charge (density)
int PostProcess(void);	// a typical post-processing function

neBEMState = 0;
int fstatus = neBEMInitialize();
assert(fstatus == 0);

// The following function uses neBEMGetNbPrimitives, neBEMGetPrimitive,
// neBEMVolumeDescription and neBEMVolumePoint, the last one used sparingly
fstatus = neBEMReadGeometry();
assert(fstatus == 0);

// Fix the level of discretization
int **elementNbs;
elementNbs = imatrix(1, NbPrimitives, 1, 2);
for(register int prim = 1; prim <= NbPrimitives; ++prim)
	{
	elementNbs[prim][1] = MaxNbElementsOnLength;
	elementNbs[prim][2] = MaxNbElementsOnLength;;
	}
fstatus = neBEMDiscretize(elementNbs);
assert(fstatus == 0);

// Set up boundary condition
fstatus = neBEMBoundaryConditions();
assert(fstatus == 0);

// Flags and any other globally accessible variable can be modified anywhere
// during execution.
NewModel = 1; NewMesh = 1; NewBC = 1; NewPP = 1;
ModelCntr = 1; MeshCntr = 1; BCCntr = 1; PPCntr = 1;

TimeStep = 1; EndOfTime = 1;

// Set up distributions with known charge / charge densities
fstatus = neBEMKnownCharges(&SpecifyKnownCharges);
assert(fstatus == 0);

fstatus = neBEMSolve();
assert(fstatus == 0);

// Post-processing can be carried out with a separate function, or a set of
// functions residing in a separate source code.
fstatus = PostProcess();
assert(fstatus == 0);

// End neBEM gracefully
fstatus = neBEMEnd();
assert(fstatus == 0);

return 0;
}	// main ends


// Assign charge / charge densities on points, lines, surfaces and volumes
// Also assign charge / charge densities on elemnts used to describe the device
// In addition, track these entitites and update their locations.
// There is one variable, TimeStep, in neBEM header file that can be handy here
int SpecifyKnownCharges(void)
{
// Assignment of known charges is allowed any time after the number of elemnts
// has been decided, the element array has been allocated and the discretization
// completed.
if(neBEMState < 4)
  {
  printf("Charge can be assigned only beyond state 4 ...\n");
  printf("returning ...\n");
  return(-1);
  }

// Assign linear or surface charge densities on the mesh
for(register unsigned int ele = 1; ele <= NbElements; ++ele)
  {
  (EleArr+ele-1)->Assigned = 0.0;
  }

// Points with known charge
NbPtsKnCh = 0;
if(NbPtsKnCh)
  {
	// allocation necessary only for the first time, or made elsewhere?
	// or, reallocate as the numnber of charged particles increase in number?
	double xpos, ypos, zpos, charge;

  PtKnChArr = (PtKnCh *) malloc(NbPtsKnCh * sizeof(PtKnCh));

  for(register unsigned int point = 1; point <= NbPtsKnCh; ++point)
    {
		xpos = 0.0; ypos = 0.0; zpos = 0.0; charge = 0.0;
    (PtKnChArr+point-1)->Nb = point;
    (PtKnChArr+point-1)->P.X = xpos;
    (PtKnChArr+point-1)->P.Y = ypos;
    (PtKnChArr+point-1)->P.Z = zpos;
    (PtKnChArr+point-1)->Assigned = charge; // this is not density
    }
  }

// The following needs to be dealt in a manner similar to the above
// Lines with known charge density
NbLinesKnCh = 0;
// Areas with known charge density
NbAreasKnCh = 0;
// Volumes with known charge density
NbVolsKnCh = 0;

return 0;
}	// end of UpdateKnownCharges
