// A simple example with dielectric interface having a finite step
// Each rectangular surface of step is represented by two triangular primitives

#include <stdio.h>
#include <assert.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>


int main(void)
{
int PostProcess(void);	// a typical post-processing function

neBEMState = 0;
int fstatus = neBEMInitialize();
assert(fstatus == 0);

// The following function uses neBEMGetNbPrimitives, neBEMGetPrimitive,
// neBEMVolumeDescription and neBEMVolumePoint, the last one used sparingly
fstatus = neBEMReadGeometry();
assert(fstatus == 0);

// Fix the level of discretization.
int **elementNbs;
elementNbs = imatrix(1, NbPrimitives, 1, 2);
for(register int prim = 1; prim <= NbPrimitives; ++prim)
	{
	elementNbs[prim][1] = MaxNbElementsOnLength;
	elementNbs[prim][2] = MaxNbElementsOnLength;
  // if(prim == 5)
    // {
    // elementNbs[prim][1] = MaxNbElementsOnLength;
    // elementNbs[prim][2] = MaxNbElementsOnLength;
    // }
	}
fstatus = neBEMDiscretize(elementNbs);
assert(fstatus == 0);

// Set up boundary condition
fstatus = neBEMBoundaryConditions();
assert (fstatus == 0);

// A flag needs to be set and transmitted to neBEMSolve to indicate whether
// the matrix inversion for this geometry has been carried out already etc.
NewModel = 1; NewMesh = 1; NewBC = 1; NewPP = 1;
ModelCntr = 1; MeshCntr = 1; BCCntr = 1; PPCntr = 1;
TimeStep = 1; EndOfTime = 1;
fstatus = neBEMSolve();
assert(fstatus == 0);

// Post-processing can be carried out with a separate function, or a set of
// functions residing in a separate source code.
fstatus = PostProcess();
assert(fstatus == 0);

// End neBEM gracefully
fstatus = neBEMEnd();
assert(fstatus == 0);

return 0;
}	// main ends
