#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>
#include <dirent.h>
#include <errno.h>

#include <NR.h>
#include <Vector.h>
#include <Device.h>

FILE *PrepareInputFiles(char DeviceInDir[], char ModelSubDir[],	// ingoing
												char DeviceInFileName[], char ModelInDir[])	// outcoming
{
// Preparations for creating the mother and daughter input files
char CmdDirStr[256];
strcpy(DeviceInFileName, DeviceInDir);
// check if this directory exists; if not, creat it
DIR *dir = opendir(DeviceInDir);
if(dir)	// directory exists
	{
	closedir(dir);
	}
else if (ENOENT == errno)
	{
	printf("In Device.c: %s does not exist and is being created ...\n");
	strcpy(ModelInDir, DeviceInDir);
	sprintf(CmdDirStr, "mkdir -p %s", ModelInDir);
	if(!system(CmdDirStr))	// returns 0 is successful
		{
    printf("Device directory created successfully.\n");
		}
	else
		{
    printf("Problem creating device directory ... exiting ...\n");
		exit(0);
		}
	}
else
	{
	printf("In Device.c: opendir failed for some other reason ...\n");
	}

strcat(DeviceInFileName, ModelSubDir);
strcat(DeviceInFileName, ".inp");
FILE *fModelInp;
fModelInp = fopen(DeviceInFileName, "w");
assert(fModelInp != NULL);

strcpy(ModelInDir, DeviceInDir);
strcat(ModelInDir, ModelSubDir);
strcat(ModelInDir, "/");
sprintf(CmdDirStr, "mkdir -p %s", ModelInDir);
if(system(CmdDirStr))  // returns 0 if successful
    {
    char option;
    printf("remove directory %s, if existing?\n", ModelInDir);
    printf("ALL DATA IN THIS DIRECTORY WILL BE LOST!\n");
    option = getc(stdin);
    if( (option == 'y') || (option == 'Y') )
        {
        sprintf(CmdDirStr, "rm -rf %s", ModelInDir);
        system(CmdDirStr);
        sprintf(CmdDirStr, "mkdir -p %s", ModelInDir);
        if(system(CmdDirStr))  // returns 0 if successful
            {
            printf("Failure to create directory ... exiting.\n");
            exit(-1);
            }
        }
    else
        {
        printf("Exiting to system ...\n");
        exit(-1);
        }
    }
else
    printf("Primitive input directory created successfully.\n");

// start writing the mother input file
// It is ok to have these initialized to zero and written into the file
// This will be corrected at the end of primitve creation
int NbPrimitives = 0;
int NbVolumes = 0;

fprintf(fModelInp, "#====>Input directory\n");
fprintf(fModelInp, "%s\n", ModelInDir);
fprintf(fModelInp, "#====>No. of primitives:\n");
fprintf(fModelInp, "%6d\n", NbPrimitives);
fprintf(fModelInp, "#====>No. of volumes:\n");
fprintf(fModelInp, "%6d\n", NbVolumes);

return(fModelInp);
}	// PrepareInputFiles ends


// Create data file for viewing device
FILE *PrepareViewingFiles(char ModelInDir[])
{
char DataFileName[256];
strcpy(DataFileName, ModelInDir); strcat(DataFileName, "DeviceData.out");
FILE *fData = fopen(DataFileName, "w");
assert(fData != NULL);

return(fData);
}


// function to create volumes
void CreateVolume(char ModelInDir[], int VolRef,
									int VolShape, int VolMaterial, int VolBoundaryType,
									double VolEpsilon, double VolPotential, double VolCharge)
{
char TmpFileName[256], VolRefInFileName[256];
char strVolRef[10];

sprintf(strVolRef, "%d", VolRef);
strcpy(TmpFileName, "Volume"); strcat(TmpFileName, strVolRef);
strcat(TmpFileName, ".inp");
strcpy(VolRefInFileName, ModelInDir);
strcat(VolRefInFileName, TmpFileName);

WriteVolRefInFile(VolRefInFileName,
                    VolShape, VolMaterial, VolEpsilon, VolPotential,
                    VolCharge, VolBoundaryType);
}	// function CreateVolume ends


// Create a polygon primitive
// Normal computation cannot work because computed normal does not necessarily
// point to the exterior of the body; Hence this has to be input by the user
void CreatePolygon(FILE *fModelInp, FILE *fData, char ModelInDir[],
										int NbPrimitives,
              			int nvertex,
										double xvert[], double yvert[], double zvert[],
										double xnorm, double ynorm, double znorm,
              			int VolRef1, int VolRef2, int NbXSegs, int NbYSegs,
										int PeriodicTypeX, int PeriodicInX, double XPeriod,
										int PeriodicTypeY, int PeriodicInY, double YPeriod,
										int PeriodicTypeZ, int PeriodicInZ, double ZPeriod)
{
char strNbPrim[10], TmpFileName[256];
sprintf(strNbPrim, "%d", NbPrimitives);
strcpy(TmpFileName, "Polygon");	// split to accommodate numbering
strcat(TmpFileName, strNbPrim);
strcat(TmpFileName, ".inp");
fprintf(fModelInp, "#Input file name for primitve:\n");
fprintf(fModelInp, "%s\n", TmpFileName);

char SurfaceInFileName[256];
strcpy(SurfaceInFileName, ModelInDir);
strcat(SurfaceInFileName, TmpFileName);
WriteSurfaceInFile(SurfaceInFileName,
										nvertex, xvert, yvert, zvert, xnorm, ynorm, znorm,
										VolRef1, VolRef2, NbXSegs, NbYSegs,
										PeriodicTypeX, PeriodicInX, XPeriod,
										PeriodicTypeY, PeriodicInY, YPeriod,
										PeriodicTypeZ, PeriodicInZ, ZPeriod);
WriteDeviceDataFile(fData, NbPrimitives, nvertex, xvert, yvert, zvert,
											PeriodicTypeX, PeriodicInX, XPeriod,
											PeriodicTypeY, PeriodicInY, YPeriod,
											PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// CreatePolygon ends


// function to create a line primitive
void CreateLine(FILE *fModelInp, FILE *fData, char ModelInDir[],
								int NbPrimitives,
								int nvertex, double xvert[], double yvert[], double zvert[],
								double radius, int VolRef1, int VolRef2, int NbSegs,
								int PeriodicTypeX, int PeriodicInX, double XPeriod,
								int PeriodicTypeY, int PeriodicInY, double YPeriod,
								int PeriodicTypeZ, int PeriodicInZ, double ZPeriod)
{
char strNbPrim[10], TmpFileName[256];
sprintf(strNbPrim, "%d", NbPrimitives);
strcpy(TmpFileName, "ThinWire");	// split to accommodate numbering
strcat(TmpFileName, strNbPrim);
strcat(TmpFileName, ".inp");
fprintf(fModelInp, "#Input file name for primitve:\n");
fprintf(fModelInp, "%s\n", TmpFileName);

char WireInFileName[256];
strcpy(WireInFileName, ModelInDir);
strcat(WireInFileName, TmpFileName);
WriteWireInFile(WireInFileName,
                nvertex,
								xvert, yvert, zvert,
                radius,
								VolRef1, VolRef2, NbSegs,
								PeriodicTypeX, PeriodicInX, XPeriod,
								PeriodicTypeY, PeriodicInY, YPeriod,
								PeriodicTypeZ, PeriodicInZ, ZPeriod);
WriteDeviceDataFile(fData, NbPrimitives, nvertex, xvert, yvert, zvert,
											PeriodicTypeX, PeriodicInX, XPeriod,
											PeriodicTypeY, PeriodicInY, YPeriod,
											PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// End of creation of line primitive


// Create the map file for computing potential and flux
void CreateMapFile(char ModelInDir[],
               			int NbMaps,	int *NbBlock,
               			int *NbXMap, int *NbYMap, int *NbZMap,
               			double *MapLX, double *MapLY, double *MapLZ,
               			double *MapXMid, double *MapYMid, double *MapZMid,
               			int NbLines, int *NbSegments,
               			double *XStart, double *YStart, double *ZStart,
										double *XStop, double *YStop, double *ZStop,
               			int NbPoints,
										double *XPoint, double *YPoint, double *ZPoint)
{
char MapInFileName[256];
strcpy(MapInFileName, ModelInDir);
strcat(MapInFileName, "MapFile.inp");	// has to tally with info in Device

WriteMapInFile(MapInFileName,
               NbMaps, NbBlock,
               NbXMap, NbYMap, NbZMap,
               MapLX, MapLY, MapLZ,
               MapXMid, MapYMid, MapZMid,
               NbLines, NbSegments,
               XStart, YStart, ZStart, XStop, YStop, ZStop,
               NbPoints, XPoint, YPoint, ZPoint);
}	// CreateMapFile ends


// function to write volume reference input files
void WriteVolRefInFile(char VolRefInFileName[],
												int VolShape, int VolMaterial, double VolEpsilon,
												double VolPotential, double VolCharge,
												int VolBoundaryType)
{
FILE *fVolRef;

fVolRef = fopen(VolRefInFileName, "w");
if(fVolRef == NULL)
  {
  printf("problem opening VolRef input file %s ... exiting ...\n",
              VolRefInFileName);
  exit(-1);
  }

fprintf(fVolRef, "#Shape of the volume:\n");
fprintf(fVolRef, "%d\n", VolShape);
fprintf(fVolRef, "#Material of the volume:\n");
fprintf(fVolRef, "%d\n", VolMaterial);
fprintf(fVolRef, "#Relative permittivity:\n");
fprintf(fVolRef, "%lg\n", VolEpsilon);
fprintf(fVolRef, "#Potential:\n");
fprintf(fVolRef, "%lg\n", VolPotential);
fprintf(fVolRef, "#Charge:\n");
fprintf(fVolRef, "%lg\n", VolCharge);
fprintf(fVolRef, "#Boundary type:\n");
fprintf(fVolRef, "%d\n", VolBoundaryType);
fclose(fVolRef);
}	// WriteVolRefInFile ends


// function to write surface primitive input files
void WriteSurfaceInFile(char SurfaceInFileName[],
												int nvertex,
												double xvert[], double yvert[], double zvert[],
												double xnorm, double ynorm, double znorm,
												int VolRef1, int VolRef2,
												int NbXSegs, int NbYSegs,
												int PeriodicTypeX, int PeriodicInX, double XPeriod,
												int PeriodicTypeY, int PeriodicInY, double YPeriod,
												int PeriodicTypeZ, int PeriodicInZ, double ZPeriod)
{
FILE *fInp;

fInp = fopen(SurfaceInFileName, "w");
if(fInp == NULL)
  {
  printf("problem opening Primitive input file %s ... exiting ...\n",
              SurfaceInFileName);
  exit(-1);
  }

fprintf(fInp, "#Number of vertices:\n");
fprintf(fInp, "%d\n", nvertex);
for(register int vertex = 0; vertex < nvertex; ++vertex)
	{
	fprintf(fInp, "#Vertex %d:\n", vertex);
	fprintf(fInp, "%lg %lg %lg\n",
										xvert[vertex], yvert[vertex], zvert[vertex]);
	}
fprintf(fInp, "#Outward normal:\n");
fprintf(fInp, "%lg %lg %lg\n", xnorm, ynorm, znorm);
fprintf(fInp, "#Volume references:\n");
fprintf(fInp, "%d %d\n", VolRef1, VolRef2);
fprintf(fInp, "#Maximum number of segments:\n");
fprintf(fInp, "%d %d\n", NbXSegs, NbYSegs);
fprintf(fInp, "#Information on X periodicity:\n");
fprintf(fInp, "%d %d %lg\n", PeriodicTypeX, PeriodicInX, XPeriod);
fprintf(fInp, "#Information on Y periodicity:\n");
fprintf(fInp, "%d %d %lg\n", PeriodicTypeY, PeriodicInY, YPeriod);
fprintf(fInp, "#Information on Z periodicity:\n");
fprintf(fInp, "%d %d %lg\n", PeriodicTypeZ, PeriodicInZ, ZPeriod);
fclose(fInp);
}	// WriteSurfaceInFile ends


// function to write wire primitive input files
void WriteWireInFile(char WireInFileName[],
											int nvertex,
											double xvert[], double yvert[], double zvert[],
											double radius,
											int VolRef1, int VolRef2, int NbSegs,
											int PeriodicTypeX, int PeriodicInX, double XPeriod,
											int PeriodicTypeY, int PeriodicInY, double YPeriod,
											int PeriodicTypeZ, int PeriodicInZ, double ZPeriod)
{
FILE *fInp;

fInp = fopen(WireInFileName, "w");
if(fInp == NULL)
  {
  printf("problem opening Primitive input file %s ... exiting ...\n",
              WireInFileName);
  exit(-1);
  }

fprintf(fInp, "#Number of vertices:\n");
fprintf(fInp, "%d\n", nvertex);
for(register int vertex = 0; vertex < 2; ++vertex)
	{
	fprintf(fInp, "#Vertex %d:\n", vertex);
	fprintf(fInp, "%lg %lg %lg\n",
										xvert[vertex], yvert[vertex], zvert[vertex]);
	}
fprintf(fInp, "#radius:\n");
fprintf(fInp, "%lg\n", radius);
fprintf(fInp, "#Volume references:\n");
fprintf(fInp, "%d %d\n", VolRef1, VolRef2);
fprintf(fInp, "#Maximum number of segments:\n");
fprintf(fInp, "%d\n", NbSegs);
fprintf(fInp, "#Information on X periodicity:\n");
fprintf(fInp, "%d %d %lg\n", PeriodicTypeX, PeriodicInX, XPeriod);
fprintf(fInp, "#Information on Y periodicity:\n");
fprintf(fInp, "%d %d %lg\n", PeriodicTypeY, PeriodicInY, YPeriod);
fprintf(fInp, "#Information on Z periodicity:\n");
fprintf(fInp, "%d %d %lg\n", PeriodicTypeZ, PeriodicInZ, ZPeriod);
fclose(fInp);
}	// WriteWireInFile ends


void WriteMapInFile(char MapInFileName[],
                    int NbMaps, int *NbBlock,
                    int *NbXMap, int *NbYMap, int *NbZMap,
                    double *MapLX, double *MapLY, double *MapLZ,
                    double *MapXMid, double *MapYMid, double *MapZMid,
                    int NbLines, int *NbSegments,
                    double *XStart, double *YStart, double *ZStart,
                    double *XStop, double *YStop, double *ZStop,
                    int NbPoints,
                    double *XPoint, double *YPoint, double *ZPoint)
{
FILE *fMap;

fMap = fopen(MapInFileName, "w");
if(fMap == NULL)
  {
  printf("problem opening Primitive input file %s ... exiting ...\n",
              MapInFileName);
  exit(-1);
  }

fprintf(fMap, "# Number of maps\n");
fprintf(fMap, "%d\n", NbMaps);
for(register int map=1; map<=NbMaps; ++map)
	{
	fprintf(fMap, "# Number of X, Y, Z points in map %d\n", map);
	fprintf(fMap, "%d %d %d\n", NbXMap[map], NbYMap[map], NbZMap[map]);
	fprintf(fMap, "# Grid lengths in X, Y, Z directions\n");
	fprintf(fMap, "%lg %lg %lg\n", MapLX[map], MapLY[map], MapLZ[map]);
	fprintf(fMap, "# Grid mid points in X, Y and Z  directions\n");
	fprintf(fMap, "%lg %lg %lg\n", MapXMid[map], MapYMid[map], MapZMid[map]);
	fprintf(fMap, "# Number of points in one block\n");
	fprintf(fMap, "%d\n", NbBlock[map]);
	}

fprintf(fMap, "# Number of lines\n");
fprintf(fMap, "%d\n", NbLines);
for(register int line=1; line<=NbLines; ++line)
	{
	fprintf(fMap, "# Start point X, Y, Z in line %d\n", line);
	fprintf(fMap, "%le %le %le\n", XStart[line], YStart[line], ZStart[line]);
	fprintf(fMap, "# Stop point X, Y, Z in line %d\n", line);
	fprintf(fMap, "%le %le %le\n", XStop[line], YStop[line], ZStop[line]);
	fprintf(fMap, "# Number of segments\n");
	fprintf(fMap, "%d\n", NbSegments[line]);
	}

fprintf(fMap, "# Number of points\n");
fprintf(fMap, "%d\n", NbPoints);
for(register int point = 1; point <= NbPoints; ++point)
	{
	fprintf(fMap, "# X\t\tY\t\tZ for point %d\n", point);
  fprintf(fMap, "%lg\t%lg\t%lg\n",
                XPoint[point], YPoint[point], ZPoint[point]);
  }

fclose(fMap);
}	// WriteMapInFile ends


// In order to ensure that repeated primitives are easily displayed, we are
// writing four blocks of coordinates per primitive, one for the real device
// and three other for possible repeatitions. If repeatitions are there, they
// will be distinctly different. If not, the data are wasted as the lines are
// drawn on top of one another. The separation is determined, naturally, by
// the values of XPeriod, YPeriod and ZPeriod.
void WriteDeviceDataFile(FILE *fData, int NbPrimitives,
                    			int nvertex,
													double xvert[], double yvert[], double zvert[],
													int PeriodicTypeX, int PeriodicInX, double XPeriod,
													int PeriodicTypeY, int PeriodicInY, double YPeriod,
													int PeriodicTypeZ, int PeriodicInZ, double ZPeriod)
{
fprintf(fData, "# Primitive Number: %d\n", NbPrimitives);
for(register int vertex = 0; vertex < nvertex; ++vertex)
	{
	fprintf(fData, "%lg %lg %lg %d\n",
									xvert[vertex], yvert[vertex], zvert[vertex], NbPrimitives);
	}
if(nvertex > 2)	// closure of polygon not necessary for a wire
	fprintf(fData, "%lg %lg %lg %d\n",
									xvert[0], yvert[0], zvert[0], NbPrimitives);
fprintf(fData, "\n");

// Takes care of a possible ZPeriod
if(PeriodicTypeZ)
	{
for(register int vertex = 0; vertex < nvertex; ++vertex)
  {
  fprintf(fData, "%lg %lg %lg %d\n",
                  xvert[vertex], yvert[vertex], zvert[vertex]+ZPeriod,
									NbPrimitives);
  }
if(nvertex > 2) // closure of polygon not necessary for a wire
  fprintf(fData, "%lg %lg %lg %d\n",
                  xvert[0], yvert[0], zvert[0]+ZPeriod, NbPrimitives);
fprintf(fData, "\n");
	}

// Takes care of a possible YPeriod
if(PeriodicTypeY)
	{
for(register int vertex = 0; vertex < nvertex; ++vertex)
  {
  fprintf(fData, "%lg %lg %lg %d\n",
                  xvert[vertex], yvert[vertex]+YPeriod, zvert[vertex],
									NbPrimitives);
  }
if(nvertex > 2) // closure of polygon not necessary for a wire
  fprintf(fData, "%lg %lg %lg %d\n",
                  xvert[0], yvert[0]+YPeriod, zvert[0], NbPrimitives);
fprintf(fData, "\n");
	}

// Takes care of a possible XPeriod
if(PeriodicTypeX)
	{
for(register int vertex = 0; vertex < nvertex; ++vertex)
  {
  fprintf(fData, "%lg %lg %lg %d\n",
                  xvert[vertex]+XPeriod, yvert[vertex], zvert[vertex],
									NbPrimitives);
  }
if(nvertex > 2) // closure of polygon not necessary for a wire
  fprintf(fData, "%lg %lg %lg %d\n",
                  xvert[0]+XPeriod, yvert[0], zvert[0], NbPrimitives);
fprintf(fData, "\n");
	}
}	// WriteDeviceDataFile ends


// End the mother input file properly
void EndDeviceInputFile(FILE *fModelInp,
												int NbBCondns, 
												char DeviceOutDir[],
												char ModelInDir[], int NbPrimitives, int NbVolumes)
{
fprintf(fModelInp, "#====>No. of boundary conditions per element\n");
fprintf(fModelInp, "%d\n", NbBCondns);
fprintf(fModelInp, "#====>Device output directory name:\n");
fprintf(fModelInp, "%s\n", DeviceOutDir);
fprintf(fModelInp, "#Map input file\n");
fprintf(fModelInp, "MapFile.inp\n");

// rewrite NbPrimitives and NbVolumes information
rewind(fModelInp);

fprintf(fModelInp, "#====>Input directory\n");
fprintf(fModelInp, "%s\n", ModelInDir);
fprintf(fModelInp, "#====>No. of primitives:\n");
fprintf(fModelInp, "%6d\n", NbPrimitives);
fprintf(fModelInp, "#====>No. of volumes:\n");
fprintf(fModelInp, "%6d\n", NbVolumes);

fclose(fModelInp);

// create Viewer files
// Note that each primitive has four blocks associated with it
// Block number starts from 0
char DataFileName[256];
strcpy(DataFileName, ModelInDir); strcat(DataFileName, "DeviceData.out");
char ViewerFileName[256];
strcpy(ViewerFileName, ModelInDir); strcat(ViewerFileName, "ViewDevice.gp");
FILE *fView = fopen(ViewerFileName, "w");
assert(fView != NULL);
fprintf(fView, "set title \"Created Device\"\n");
fprintf(fView, "unset key\n");
fprintf(fView, "set xlabel \"X(m)\"\n");
fprintf(fView, "set ylabel \"Y(m)\"\n");
fprintf(fView, "set zlabel \"Z(m)\"\n");
fprintf(fView, "splot \\\n");
for(register int prim = 1; prim < NbPrimitives; ++prim)
	{
	fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (prim-1)*4, (prim-1)*4);
	fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (prim-1)*4 + 1, (prim-1)*4 + 1);
	fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (prim-1)*4 + 2, (prim-1)*4 + 2);
	fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (prim-1)*4 + 3, (prim-1)*4 + 3);
	}
fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (NbPrimitives-1)*4, (NbPrimitives-1)*4);
fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (NbPrimitives-1)*4 + 1, (NbPrimitives-1)*4 + 1);
fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (NbPrimitives-1)*4 + 2, (NbPrimitives-1)*4 + 2);
fprintf(fView, "\'%s\' ev :::%d::%d w l\n",
									DataFileName, (NbPrimitives-1)*4 + 3, (NbPrimitives-1)*4 + 3);
fprintf(fView, "pause -1");
fclose(fView);
printf("Issue \"gnuplot %s\" to visualize the complete device created.\n",
				ViewerFileName);

strcpy(ViewerFileName, ModelInDir);
strcat(ViewerFileName, "ViewComponents.gp");
fView = fopen(ViewerFileName, "w");
assert(fView != NULL);
fprintf(fView, "unset key\n");
fprintf(fView, "set xlabel \"X(m)\"\n");
fprintf(fView, "set ylabel \"Y(m)\"\n");
fprintf(fView, "set zlabel \"Z(m)\"\n");
for(register int prim = 1; prim <= NbPrimitives; ++prim)
	{
	fprintf(fView, "set title \"Component View: %d\"\n", prim);
	fprintf(fView, "splot \\\n");
	for(register int component = 1; component < prim; ++component)
		{
		fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (component-1)*4, (component-1)*4);
		fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (component-1)*4 + 1, (component-1)*4 + 1);
		fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (component-1)*4 + 2, (component-1)*4 + 2);
		fprintf(fView, "\'%s\' ev :::%d::%d w l, \\\n",
									DataFileName, (component-1)*4 + 3, (component-1)*4 + 3);
		}

	fprintf(fView, "\'%s\' ev :::%d::%d w l lw 2, \\\n",
									DataFileName, (prim-1)*4, (prim-1)*4);
	fprintf(fView, "\'%s\' ev :::%d::%d w l lw 2, \\\n",
									DataFileName, (prim-1)*4 + 1, (prim-1)*4 + 1);
	fprintf(fView, "\'%s\' ev :::%d::%d w l lw 2, \\\n",
									DataFileName, (prim-1)*4 + 2, (prim-1)*4 + 2);
	fprintf(fView, "\'%s\' ev :::%d::%d w l lw 2\n",
									DataFileName, (prim-1)*4 + 3, (prim-1)*4 + 3);
	fprintf(fView, "pause -1\n\n");
	}
fclose(fView);
printf("Issue \"gnuplot %s\" to visualize the device components created.\n",
				ViewerFileName);
}
