// An example that creates one plate.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>

#include <NR.h>
#include <Vector.h>
#include <Device.h>


int main(void)
{
// User inputs - can be read in from an input file or hash-defined, if desired.
// Or, even better, for a truly elaborate device, use a separate input file.
// Another option could be to have ".def" file which is #included at the
// beginning of a device code, as has been done in the micromegas device codes.
// The convenience of using separate def or input files is to avoid scrolling
// too often. There can, of course, be many such files possibly representing
// each component of a device.
char DeviceInDirName[256], DeviceInFileName[256],
			ModelName[256], DeviceOutDirName[256];
strcpy(DeviceInDirName, "/home/supratik/neBEMWD/Inputs/OnePlateBy2Tri/");	// existing
strcpy(ModelName, "Model1");	// do NOT put an appending /
strcpy(DeviceOutDirName, "/home/supratik/neBEMWD/Outputs/OnePlateBy2Tri/");
strcat(DeviceOutDirName, ModelName);
strcat(DeviceOutDirName, "/");

// preparation of necessary input files
char DeviceInDir[256], ModelSubDir[256], ModelInDir[256];
FILE *fModelInp;
strcpy(DeviceInDir, DeviceInDirName);	// existing directory
strcpy(ModelSubDir, ModelName);	// to be created
fModelInp = PrepareInputFiles(DeviceInDir, ModelSubDir,
															DeviceInFileName, ModelInDir);

// file-handler for device data file used for viewing
FILE *fData;
fData = PrepareViewingFiles(ModelInDir);

// Create volume reference data
// The surfaces that define the device are interfaces between the volumes
// void CreateVolume(char ModelInDir[], int VolRef,
//                   int VolShape, int VolMaterial, int VolBoundaryType,
//                   double VolEpsilon, double VolPotential, double VolCharge);
int NbVolumes = 0;

{
// Gas: VolRef1
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 0, 0, 4, 1.0, 0.0, 0.0);

// Conductor 1: VolRef2
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, 1.0, 0.0);
}	// Creation of volume reference data files over.


// Start building the device using primitives 
int NbPrimitives = 0;

// Component 1 - a triangluar plate
{

// a triangular plate
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[3], yvert[3], zvert[3];
double xnorm, ynorm, znorm;
double SurfX = 0.0, SurfY = 0.0, SurfZ = 0.0;	// CG of the combined square plate
double LengthX = 1.0, LengthY = 1.0;	// length and breadth
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

nvertex = 3; VolRef1 = 2; VolRef2 = 1; NbXSegs = 1; NbYSegs = 1;

// compute vertices
xvert[0] = SurfX+0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX-0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// first triangularPlate ends

}	// Component 1 ends

// Component 2 - second triangluar plate
{

// another triangular plate
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[3], yvert[3], zvert[3];
double xnorm, ynorm, znorm;
double SurfX = 0.0, SurfY = 0.0, SurfZ = 0.0;	// CG of the combined square plate
double LengthX = 1.0, LengthY = 1.0;	// length and breadth
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

nvertex = 3; VolRef1 = 2; VolRef2 = 1; NbXSegs = 1; NbYSegs = 1;

// compute vertices
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY+0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY-0.5*LengthY; zvert[2] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// another triangularPlate ends

} // Component 2 ends


// Close device data file
fclose(fData);


// Creation of map file to evaluate properties on maps (2d grids), along lines
// and at given points
{
// data that goes into the Map files
int NbMaps, NbLines, NbPoints;
int *NbXMap, *NbYMap, *NbZMap;
double *MapLX, *MapLY, *MapLZ;
double *MapXMid, *MapYMid, *MapZMid;
int *NbBlock;
int *NbSegments;
double *XStart, *YStart, *ZStart, *XStop, *YStop, *ZStop;
double *XPoint, *YPoint, *ZPoint;

// Maps - there can be multiple maps
NbMaps = 0;
if(NbMaps)
	{
	NbXMap = ivector(1, NbMaps); NbYMap = ivector(1, NbMaps);
	NbZMap = ivector(1, NbMaps); MapLX = dvector(1, NbMaps);
	MapLY = dvector(1, NbMaps); MapLZ = dvector(1, NbMaps);
	MapXMid = dvector(1, NbMaps); MapYMid = dvector(1, NbMaps);
	MapZMid = dvector(1, NbMaps); NbBlock = ivector(1, NbMaps);

	// Map 1
	NbXMap[1] = 21; NbYMap[1] = 1; NbZMap[1] = 21; // points along each axis
	MapLX[1] = 1.0; MapLY[1] = 0.0; MapLZ[1] = 0.2;	// lengths of the map
	MapXMid[1] = 0.0; MapYMid[1] = 0.0; MapZMid[1] = 0.0;	// mid-points
	NbBlock[1] = NbZMap[1];	// length of each block (in relation to gnuplot)
	}

// Lines - can be multiple lines
NbLines = 2;
if(NbLines)
	{
	NbSegments = ivector(1, NbLines); XStart = dvector(1, NbLines);
	YStart = dvector(1, NbLines); ZStart = dvector(1, NbLines);
	XStop = dvector(1, NbLines); YStop = dvector(1, NbLines);
	ZStop = dvector(1, NbLines);

	// Line 1 - a line perpendicular to the plate, through its origin
	XStart[1] = 0.0; YStart[1] = 0.0; ZStart[1] = -0.1;
	XStop[1] = 0.0; YStop[1] = 0.0; ZStop[1] = 0.1;
	NbSegments[1] = 10;

	// Line 2 - a line along X axis, just one micron above the plate
	XStart[2] = -1.0; YStart[2] = 0.0; ZStart[2] = 1.0e-6;
	XStop[2] = 1.0; YStop[2] = 0.0; ZStop[2] = 1.0e-6;
	NbSegments[2] = 10;
	}

// Points - can be multiple points
NbPoints = 3;
if(NbPoints)
  {
  XPoint = dvector(1, NbPoints); YPoint = dvector(1, NbPoints);
  ZPoint = dvector(1, NbPoints);

  for(register int point = 1; point <= NbPoints; ++point)
    {
    XPoint[point] = 0.0; YPoint[point] = 0.0; ZPoint[point] = 0.0;
		if(point == 1) ZPoint[point] = -0.1;
		if(point == 3) ZPoint[point] = +0.1;
    }
  }

CreateMapFile(ModelInDir,
               NbMaps, NbBlock,
               NbXMap, NbYMap, NbZMap,
               MapLX, MapLY, MapLZ,
               MapXMid, MapYMid, MapZMid,
               NbLines, NbSegments,
               XStart, YStart, ZStart, XStop, YStop, ZStop,
               NbPoints, XPoint, YPoint, ZPoint);
}	// Map creation ends


// Complete the DeviceInputFile
int NbBCondns = 1;
EndDeviceInputFile(fModelInp, NbBCondns,
										DeviceOutDirName, ModelInDir, NbPrimitives, NbVolumes);

return 0;
}	// main ends
