// Calendered crossings are not modelled - rest of the mesh made of cylinders

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>

#include <NR.h>
#include <Vector.h>
#include <Device.h>


int main(void)
{
#include "CylCal1.def"

// preparation of necessary input files
char DeviceInDir[256], DeviceInFileName[256], ModelSubDir[256], ModelInDir[256];
FILE *fModelInp;
strcpy(DeviceInDir, DeviceInDirName);	// existing directory
strcpy(ModelSubDir, ModelName);	// to be created
fModelInp = PrepareInputFiles(DeviceInDir, ModelSubDir,
                              DeviceInFileName, ModelInDir);

// file-handler for device data file used for viewing
FILE *fData;
fData = PrepareViewingFiles(ModelInDir);

// Create volume reference data
// For this analysis, there are five volumes
// Anode pad conductor surface at the interface of conductor and gas volumes
// Anode pad substrate surface at the interface of dielectric and gas volumes
// Cathode mesh surface at the interface of conductor and gas volumes 
// Drift cathode surface at the interface of conductor and gas volumes 
// void CreateVolume(char ModelInDir[], int VolRef,
//                   int VolShape, int VolMaterial, int VolBoundaryType,
//                   double VolEpsilon, double VolPotential, double VolCharge);
int NbVolumes = 0;	// to make it visible to EndDeviceInputFile function

{
// Gas: VolRef1
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 0, 0, 4, 1.0, 0.0, 0.0);

// Anode conductor: VolRef2
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, 0.0, 0.0);

// Dielectric: VolRef3
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 11, 4, 5.0, 0.0, 0.0);

// Cathode mesh wire: VolRef4
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, -500.0, 0.0);

// Drift surface conductor: VolRef5
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, -501.0, 0.0); // 20 V/cm drift field for 500mu d gap
}	// Creation of volume reference data files over.


// Start building the device using primitives 
int NbPrimitives = 0;

// Anode - a conducting pad and two substrate strip limbs: rectangular shapes
{

// AnodeConductingPad
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 2; VolRef2 = 1; NbXSegs = BigNb; NbYSegs = BigNb;
SurfX = XAnodePad; SurfY = YAnodePad; SurfZ = ZAnodePad;
LengthX = LXAnodePad; LengthY = LYAnodePad;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// AnodeConductingPad ends

} // Anode ends


// Cathode mesh made of 4 conducting parallelpipeds having 1/2 length of each
// hole side and the calendered portion that is represented here as rectangular
// parallelpiped, edges of which are one MeshWireRadius long
//     _____
//     | | |         ^ Y
//     | | |         | 
// |-----------|     |
// |---|///|---|     ------> X
// |-----------|
//     | | |    
//     | | |    
//     -----
{
// parallelpiped parallel to Y placed on +Y
{
// X+ surface parallel to Y
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
LengthY = 0.5*LYMeshHole; LengthZ = 2.0*MeshWireRadius;
SurfX = MeshWireRadius; SurfY = MeshWireRadius+(LYMeshHole/4.0); SurfZ = ZWireMesh;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = 1.0; ynorm = 0.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// X+ surface 

// X- surface parallel to Y
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
LengthY = 0.5*LYMeshHole; LengthZ = 2.0*MeshWireRadius;
SurfX = -MeshWireRadius; SurfY = MeshWireRadius+(LYMeshHole/4.0);
SurfZ = ZWireMesh;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = -1.0; ynorm = 0.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// X- surface

// Top Z surface of parallelpiped parallel to Y
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = 0.0; SurfY = MeshWireRadius+(LYMeshHole/4.0); SurfZ = ZWireMesh+MeshWireRadius;
LengthX = 2.0*MeshWireRadius; LengthY = 0.5*LYMeshHole;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Top Z surface

// Bottom Z surface of parallelpiped parallel to Y
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = 0.0; SurfY = MeshWireRadius+(LYMeshHole/4.0); SurfZ = ZWireMesh-MeshWireRadius;
LengthX = 2.0*MeshWireRadius; LengthY = 0.5*LYMeshHole;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Bottom Z surface 
} // parallelpiped parallel to Y placed on +Y ends

// parallelpiped parallel to Y placed on -Y
{
// X+ surface parallel to Y
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
LengthY = 0.5*LYMeshHole; LengthZ = 2.0*MeshWireRadius;
SurfX = MeshWireRadius; SurfY = -MeshWireRadius-(LYMeshHole/4.0); SurfZ = ZWireMesh;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = 1.0; ynorm = 0.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// X+ surface 

// X- surface parallel to Y
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
LengthY = 0.5*LYMeshHole; LengthZ = 2.0*MeshWireRadius;
SurfX = -MeshWireRadius; SurfY = -MeshWireRadius-(LYMeshHole/4.0); SurfZ = ZWireMesh;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = -1.0; ynorm = 0.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// X- surface

// Top Z surface of parallelpiped parallel to Y
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = 0.0; SurfY = -MeshWireRadius-(LYMeshHole/4.0); SurfZ = ZWireMesh+MeshWireRadius;
LengthX = 2.0*MeshWireRadius; LengthY = 0.5*LYMeshHole;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Top Z surface

// Bottom Z surface of parallelpiped parallel to Y
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = 0.0; SurfY = -MeshWireRadius-(LYMeshHole/4.0); SurfZ = ZWireMesh-MeshWireRadius;
LengthX = 2.0*MeshWireRadius; LengthY = 0.5*LYMeshHole;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Bottom Z surface 
} // parallelpiped parallel to Y placed on -Y ends

// parallelpiped parallel to X placed on +X
{
// Y+ surface parallel to X
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
LengthX = 0.5*LXMeshHole; LengthZ = 2.0*MeshWireRadius;
SurfX = MeshWireRadius+(LXMeshHole/4.0); SurfY = MeshWireRadius; SurfZ = ZWireMesh;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = 0.0; ynorm = 1.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Y+ surface 

// Y- surface parallel to X
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
LengthX = 0.5*LXMeshHole; LengthZ = 2.0*MeshWireRadius;
SurfX = MeshWireRadius+(LXMeshHole/4.0); SurfY = -MeshWireRadius; SurfZ = ZWireMesh;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = 0.0; ynorm = -1.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Y- surface 

// Top Z surface of parallelpiped parallel to X
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = MeshWireRadius+(LXMeshHole/4.0); SurfY = 0.0; SurfZ = ZWireMesh+MeshWireRadius;
LengthX = 0.5*LXMeshHole; LengthY = 2.0*MeshWireRadius;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Top Z surface

// Bottom Z surface of parallelpiped parallel to X
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = MeshWireRadius+(LXMeshHole/4.0); SurfY = 0.0; SurfZ = ZWireMesh-MeshWireRadius;
LengthX = 0.5*LXMeshHole; LengthY = 2.0*MeshWireRadius;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Bottom Z surface
} // parallelpiped parallel to X placed on +X ends

// parallelpiped parallel to X placed on -X
{
// Y+ surface parallel to X
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
LengthX = 0.5*LXMeshHole; LengthZ = 2.0*MeshWireRadius;
SurfX = -MeshWireRadius-(LXMeshHole/4.0); SurfY = MeshWireRadius; SurfZ = ZWireMesh;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = 0.0; ynorm = 1.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Y+ surface 

// Y- surface parallel to X
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
LengthX = 0.5*LXMeshHole; LengthZ = 2.0*MeshWireRadius;
SurfX = -MeshWireRadius-(LXMeshHole/4.0); SurfY = -MeshWireRadius; SurfZ = ZWireMesh;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = 0.0; ynorm = -1.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Y- surface 

// Top Z surface of parallelpiped parallel to X
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = -MeshWireRadius-(LXMeshHole/4.0); SurfY = 0.0; SurfZ = ZWireMesh+MeshWireRadius;
LengthX = 0.5*LXMeshHole; LengthY = 2.0*MeshWireRadius;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Top Z surface

// Bottom Z surface of parallelpiped parallel to X
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = -MeshWireRadius-(LXMeshHole/4.0); SurfY = 0.0; SurfZ = ZWireMesh-MeshWireRadius;
LengthX = 0.5*LXMeshHole; LengthY = 2.0*MeshWireRadius;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Bottom Z surface
} // parallelpiped parallel to X placed on -X ends

// Calendered part in the middle is represented as a rectangular parallelpiped.
// It is open-ended perpendicular to both X and Y, and thus has only the
// top Z and bottom Z surfaces.
// Top Z surface of the calendered portion
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = 0.0; SurfY = 0.0; SurfZ = ZWireMesh + MeshWireRadius;
LengthX = 2.0*MeshWireRadius; LengthY = 2.0*MeshWireRadius;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Top Z surface of the calendered portion

// Bottom Z surface of the calendered portion
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = SmallNb; NbYSegs = SmallNb;
SurfX = 0.0; SurfY = 0.0; SurfZ = ZWireMesh - MeshWireRadius;
LengthX = 2.0*MeshWireRadius; LengthY = 2.0*MeshWireRadius;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat;
PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX+0.5*LengthX; yvert[1] = SurfY-0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX-0.5*LengthX; yvert[3] = SurfY+0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Bottom Z surface of the calendered portion
}	// Cathode mesh ends


// Drift plane - a conductor
{

// Drift plane
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 5; VolRef2 = 1; NbXSegs = BigNb; NbYSegs = BigNb;
SurfX = XDriftPlane; SurfY = YDriftPlane; SurfZ = ZDriftPlane;
LengthX = LXDriftPlane; LengthY = LYDriftPlane;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Drift plane ends

}	// Drift

// Close device data file
fclose(fData);


// Creation of map file to evaluate properties on maps (2d grids), along lines
// and at given points
{
// data that goes into the Map files
int NbMaps, NbLines, NbPoints;
int *NbXMap, *NbYMap, *NbZMap;
double *MapLX, *MapLY, *MapLZ;
double *MapXMid, *MapYMid, *MapZMid;
int *NbBlock;
int *NbSegments;
double *XStart, *YStart, *ZStart, *XStop, *YStop, *ZStop;
double *XPoint, *YPoint, *ZPoint;

// Maps - there can be multiple maps
NbMaps = 0;
if(NbMaps)
  {
  NbXMap = ivector(1, NbMaps); NbYMap = ivector(1, NbMaps);
  NbZMap = ivector(1, NbMaps); MapLX = dvector(1, NbMaps);
  MapLY = dvector(1, NbMaps); MapLZ = dvector(1, NbMaps);
  MapXMid = dvector(1, NbMaps); MapYMid = dvector(1, NbMaps);
  MapZMid = dvector(1, NbMaps); NbBlock = ivector(1, NbMaps);

  // Map 1 - XY map in the middle of the amplificaion gap
  NbXMap[1] = 50; NbYMap[1] = 50; NbZMap[1] = 1; // points along each axis
  MapLX[1] = 3.0*(LXMeshHole + 2.0*MeshWireRadius);
  MapLY[1] = 3.0*(LYMeshHole + 2.0*MeshWireRadius);
  MapLZ[1] = 0.0; // lengths of the map
  MapXMid[1] = 0.5*LXPitch; MapYMid[1] = 0.5*LYPitch;
  MapZMid[1] = 0.5*AmplificationGap;  // mid-points
  NbBlock[1] = NbYMap[1]; // length of each block (in relation to gnuplot)

  // Map 2 - XY map one diameter below the mesh wire
  NbXMap[2] = 50; NbYMap[2] = 50; NbZMap[2] = 1; // points along each axis
  MapLX[2] = 3.0*(LXMeshHole + 2.0*MeshWireRadius);
  MapLY[2] = 3.0*(LYMeshHole + 2.0*MeshWireRadius);
  MapLZ[2] = 0.0; // lengths of the map
  MapXMid[2] = 0.5*LXPitch; MapYMid[2] = 0.5*LYPitch;
  MapZMid[2] = AmplificationGap-2.0*MeshWireRadius;  // mid-points
  NbBlock[2] = NbYMap[2]; // length of each block (in relation to gnuplot)

  // Map 3 - XY map one diameter above the mesh wire
  NbXMap[3] = 50; NbYMap[3] = 50; NbZMap[3] = 1; // points along each axis
  MapLX[3] = 3.0*(LXMeshHole + 2.0*MeshWireRadius);
  MapLY[3] = 3.0*(LYMeshHole + 2.0*MeshWireRadius);
  MapLZ[3] = 0.0; // lengths of the map
  MapXMid[3] = 0.5*LXPitch; MapYMid[3] = 0.5*LYPitch;
  MapZMid[3] = AmplificationGap+2.0*MeshWireRadius+2.0*MeshWireRadius;  // mid-points
  NbBlock[3] = NbYMap[3]; // length of each block (in relation to gnuplot)

  // Map 4 - XZ map - note change of subscript from 4 to 1!
  NbXMap[4] = 50; NbYMap[4] = 1; NbZMap[4] = 50; // points along each axis
  MapLX[4] = 3.0*(LXMeshHole + 2.0*MeshWireRadius);
  MapLY[4] = 0.0;
  MapLZ[4] = AmplificationGap+2.0*MeshWireRadius+DriftGap;  // lengths of map
  MapXMid[4] = 0.5*LXPitch; MapYMid[4] = 0.5*LYPitch;
  MapZMid[4] = 0.5*MapLZ[4];  // mid-points
  NbBlock[4] = NbZMap[4]; // length of each block (in relation to gnuplot)
  }

// Lines - can be multiple lines
NbLines = 5;
if(NbLines)
  {
  NbSegments = ivector(1, NbLines); XStart = dvector(1, NbLines);
  YStart = dvector(1, NbLines); ZStart = dvector(1, NbLines);
  XStop = dvector(1, NbLines); YStop = dvector(1, NbLines);
  ZStop = dvector(1, NbLines);

  // Line 1 - along Z through a hole center
  XStart[1] = 0.5*LXPitch; YStart[1] = 0.5*LYPitch; ZStart[1] = 0.0;
  XStop[1] = 0.5*LXPitch; YStop[1] = 0.5*LYPitch; ZStop[1] = ZDriftPlane;
  NbSegments[1] = 100;

  // Line 2 - along X through the middle of the amplification gap
  XStart[2] = -LXPitch;
  YStart[2] = 0.0;
  ZStart[2] = ZWireMesh;
  XStop[2] = LXPitch;
  YStop[2] = 0.0;
  ZStop[2] = ZWireMesh;
  NbSegments[2] = 100;

  // Line 3 - along X above the wire mesh
  XStart[3] = -LXPitch;
  YStart[3] = 0.0;
  ZStart[3] = ZWireMesh + 2.0*MeshWireRadius;
  XStop[3] = LXPitch;
  YStop[3] = 0.0;
  ZStop[3] = ZWireMesh + 2.0*MeshWireRadius;
  NbSegments[3] = 100;

  // Line 4 - along X through the middle of the amplification gap
  XStart[4] = 0.0;
  YStart[4] = -LYPitch;
  ZStart[4] = ZWireMesh;
  XStop[4] = 0.0;
  YStop[4] = LYPitch;
  ZStop[4] = ZWireMesh;
  NbSegments[4] = 100;

  // Line 5 - along X through the middle of the amplification gap
  XStart[5] = 0.0;
  YStart[5] = -LYPitch;
  ZStart[5] = ZWireMesh + 2.0*MeshWireRadius;
  XStop[5] = 0.0;
  YStop[5] = LYPitch;
  ZStop[5] = ZWireMesh + 2.0*MeshWireRadius;
  NbSegments[5] = 100;
  }

// Points - can be multiple points
NbPoints = 4;
if(NbPoints)
  {
  XPoint = dvector(1, NbPoints); YPoint = dvector(1, NbPoints);
  ZPoint = dvector(1, NbPoints);

  for(register int point = 1; point <= NbPoints; ++point)
    {
    if(point == 1)
			{XPoint[point] = 0.0; YPoint[point] = 0.0; ZPoint[point] = 137.0e-6;}
    if(point == 2)
			{XPoint[point] = 31.5e-6; YPoint[point] = 0.0; ZPoint[point] = 137.0e-6;}
    if(point == 3)
			{XPoint[point] = 0.0; YPoint[point] = 31.5e-6; ZPoint[point] = 137.0e-6;}
    if(point == 4)
			{XPoint[point] = 31.5e-6; YPoint[point] = 31.5e-6; ZPoint[point] = 137.0e-6;}
    }
  }

CreateMapFile(ModelInDir,
               NbMaps, NbBlock,
               NbXMap, NbYMap, NbZMap,
               MapLX, MapLY, MapLZ,
               MapXMid, MapYMid, MapZMid,
               NbLines, NbSegments,
               XStart, YStart, ZStart, XStop, YStop, ZStop,
               NbPoints, XPoint, YPoint, ZPoint);
}	// Map creation ends


// Complete the DeviceInputFile
int NbBCondns = 1;
EndDeviceInputFile(fModelInp, NbBCondns,
										DeviceOutDirName, ModelInDir, NbPrimitives, NbVolumes);

return 0;
}	// main ends
