// Same as D51, but multiple thin plates to represent mesh instead of one

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>

#include <NR.h>
#include <Vector.h>
#include <Device.h>


int main(void)
{
#include "MultipleThinPlate.def"

// preparation of necessary input files
char DeviceInDir[256], DeviceInFileName[256], ModelSubDir[256], ModelInDir[256];
FILE *fModelInp;
strcpy(DeviceInDir, DeviceInDirName);	// existing directory
strcpy(ModelSubDir, ModelName);	// to be created
fModelInp = PrepareInputFiles(DeviceInDir, ModelSubDir,
                              DeviceInFileName, ModelInDir);

// file-handler for device data file used for viewing
FILE *fData;
fData = PrepareViewingFiles(ModelInDir);

// Create volume reference data
// For this analysis, there are five volumes
// Anode pad conductor surface at the interface of conductor and gas volumes
// Anode pad substrate surface at the interface of dielectric and gas volumes
// Cathode mesh surface at the interface of conductor and gas volumes 
// Drift cathode surface at the interface of conductor and gas volumes 
// void CreateVolume(char ModelInDir[], int VolRef,
//                   int VolShape, int VolMaterial, int VolBoundaryType,
//                   double VolEpsilon, double VolPotential, double VolCharge);
int NbVolumes = 0;	// to make it visible to EndDeviceInputFile

{
// Gas: VolRef1
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 0, 0, 4, 1.0, 0.0, 0.0);

// Anode conductor: VolRef2
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, 0.0, 0.0);

// Dielectric: VolRef3
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 11, 4, 5.0, 0.0, 0.0);

// Cathode mesh conductor (multiple thin conducting plates): VolRef4
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, -500.0, 0.0);

// Drift surface conductor: VolRef5
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, -502.0, 0.0);
}	// Creation of volume reference data files over.


// Start building the device using primitives 
int NbPrimitives = 0;

// Anode - a conducting pad and two substrate strip limbs: rectangular shapes
{

// AnodeConductingPad
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 2; VolRef2 = 1; NbXSegs = 21; NbYSegs = 21;
SurfX = XAnodePad; SurfY = YAnodePad; SurfZ = ZAnodePad;
LengthX = LXAnodePad; LengthY = LYAnodePad;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// AnodeConductingPad ends

// AnodeStripXlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 3; VolRef2 = 1; NbXSegs = 21; NbYSegs = 21;
SurfX = XStripXlimb; SurfY = YStripXlimb; SurfZ = ZStripXlimb;
LengthX = LXStripXlimb; LengthY = LYStripXlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// AnodeStripXlimb ends

// AnodeStripYlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 3; VolRef2 = 1; NbXSegs = 21; NbYSegs = 21;
SurfX = XStripYlimb; SurfY = YStripYlimb; SurfZ = ZStripYlimb;
LengthX = LXStripYlimb; LengthY = LYStripYlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// AnodeStripYlimb ends

} // Anode ends


// Cathode mesh made of 10 conducting plates
{

// Making of the Xlimb
// YpMeshXlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = 5; NbYSegs = 5;
SurfX = XYpMeshXlimb; SurfY = YYpMeshXlimb; SurfZ = ZYpMeshXlimb;
LengthX = LXYpMeshXlimb; LengthY = LYYpMeshXlimb; LengthZ = LZYpMeshXlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = 0.0; ynorm = 1.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// YpMeshXlimb ends

// YmMeshXlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = 5; NbYSegs = 21;
SurfX = XYmMeshXlimb; SurfY = YYmMeshXlimb; SurfZ = ZYmMeshXlimb;
LengthX = LXYmMeshXlimb; LengthY = LYYmMeshXlimb; LengthZ = LZYmMeshXlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY; zvert[1] = SurfZ+0.5*LengthZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY; zvert[3] = SurfZ-0.5*LengthZ;
xnorm = 0.0; ynorm = -1.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// YmMeshXlimb ends

// ZpMeshXlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = 21; NbYSegs = 5;
SurfX = XZpMeshXlimb; SurfY = YZpMeshXlimb; SurfZ = ZZpMeshXlimb;
LengthX = LXZpMeshXlimb; LengthY = LYZpMeshXlimb; LengthZ = LXZpMeshXlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// ZpMeshXlimb ends

// ZmMeshXlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = 21; NbYSegs = 5;
SurfX = XZmMeshXlimb; SurfY = YZmMeshXlimb; SurfZ = ZZmMeshXlimb;
LengthX = LXZmMeshXlimb; LengthY = LYZmMeshXlimb; LengthZ = LXZmMeshXlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// ZmMeshXlimb ends

// Making of the Ylimb
// XpMeshYlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = 5; NbYSegs = 5;
SurfX = XXpMeshYlimb; SurfY = YXpMeshYlimb; SurfZ = ZXpMeshYlimb;
LengthX = LXXpMeshYlimb; LengthY = LYXpMeshYlimb; LengthZ = LZXpMeshYlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ-0.5*LengthZ;
xvert[2] = SurfX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ+0.5*LengthZ;
xnorm = 1.0; ynorm = 0.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// XpMeshYlimb ends

// XmMeshYlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = 21; NbYSegs = 5;
SurfX = XXmMeshYlimb; SurfY = YXmMeshYlimb; SurfZ = ZXmMeshYlimb;
LengthX = LXXmMeshYlimb; LengthY = LYXmMeshYlimb; LengthZ = LZXmMeshYlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ-0.5*LengthZ;
xvert[1] = SurfX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ-0.5*LengthZ;
xvert[2] = SurfX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ+0.5*LengthZ;
xvert[3] = SurfX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ+0.5*LengthZ;
xnorm = -1.0; ynorm = 0.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// XmMeshYlimb ends

// ZpMeshYlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = 21; NbYSegs = 21;
SurfX = XZpMeshYlimb; SurfY = YZpMeshYlimb; SurfZ = ZZpMeshYlimb;
LengthX = LXZpMeshYlimb; LengthY = LYZpMeshYlimb; LengthZ = LZZpMeshYlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// ZpMeshYlimb ends

// ZmMeshYlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY, LengthZ;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 4; VolRef2 = 1; NbXSegs = 21; NbYSegs = 21;
SurfX = XZmMeshYlimb; SurfY = YZmMeshYlimb; SurfZ = ZZmMeshYlimb;
LengthX = LXZmMeshYlimb; LengthY = LYZmMeshYlimb; LengthZ = LZZmMeshYlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// ZmMeshYlimb ends

} // Cathode mesh ends

// Drift plane - a conductor
{

// Drift plane
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 5; VolRef2 = 1; NbXSegs = 21; NbYSegs = 21;
SurfX = XDriftPlane; SurfY = YDriftPlane; SurfZ = ZDriftPlane;
LengthX = LXDriftPlane; LengthY = LYDriftPlane;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
              NbPrimitives,
              nvertex, xvert, yvert, zvert,
              xnorm, ynorm, znorm,
              VolRef1, VolRef2, NbXSegs, NbYSegs, 
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Drift plane ends

}	// Drift

// Close device data file
fclose(fData);


// Creation of map file to evaluate properties on maps (2d grids), along lines
// and at given points
{
// data that goes into the Map files
int NbMaps, NbLines, NbPoints;
int *NbXMap, *NbYMap, *NbZMap;
double *MapLX, *MapLY, *MapLZ;
double *MapXMid, *MapYMid, *MapZMid;
int *NbBlock;
int *NbSegments;
double *XStart, *YStart, *ZStart, *XStop, *YStop, *ZStop;
double *XPoint, *YPoint, *ZPoint;

// Maps - there can be multiple maps
NbMaps = 2;
if(NbMaps)
	{
	NbXMap = ivector(1, NbMaps); NbYMap = ivector(1, NbMaps);
	NbZMap = ivector(1, NbMaps); MapLX = dvector(1, NbMaps);
	MapLY = dvector(1, NbMaps); MapLZ = dvector(1, NbMaps);
	MapXMid = dvector(1, NbMaps); MapYMid = dvector(1, NbMaps);
	MapZMid = dvector(1, NbMaps); NbBlock = ivector(1, NbMaps);

	// Map 1
	NbXMap[1] = 100; NbYMap[1] = 100; NbZMap[1] = 1; // points along each axis
	MapLX[1] = LXMeshHole + WidthMeshYlimb;
	MapLY[1] = LYMeshHole + WidthMeshXlimb;
	MapLZ[1] = 0.0;	// lengths of the map
	MapXMid[1] = 0.0; MapYMid[1] = 0.0;
	MapZMid[1] = 0.5*AmplificationGap;	// mid-points
	NbBlock[1] = NbYMap[1];	// length of each block (in relation to gnuplot)

	// Map 2
	NbXMap[2] = 100; NbYMap[2] = 1; NbZMap[2] = 100; // points along each axis
	MapLX[2] = LXMeshHole + WidthMeshYlimb;
	MapLY[2] = 0.0;
	MapLZ[2] = 2.0*AmplificationGap;	// lengths of the map
	MapXMid[2] = 0.0; MapYMid[2] = 0.0;
	MapZMid[2] = 0.5*AmplificationGap;	// mid-points
	NbBlock[2] = NbZMap[2];	// length of each block (in relation to gnuplot)
	}

// Lines - can be multiple lines
NbLines = 2;
if(NbLines)
	{
	NbSegments = ivector(1, NbLines); XStart = dvector(1, NbLines);
	YStart = dvector(1, NbLines); ZStart = dvector(1, NbLines);
	XStop = dvector(1, NbLines); YStop = dvector(1, NbLines);
	ZStop = dvector(1, NbLines);

	// Line 1
	XStart[1] = 0.0; YStart[1] = 0.0; ZStart[1] = 0.0;
	XStop[1] = 0.0; YStop[1] = 0.0; ZStop[1] = ZDriftPlane;
	NbSegments[1] = 1000;

	// Line 2
	XStart[2] = -(LXMeshHole + WidthMeshYlimb);
	YStart[2] = 0.0;
	ZStart[2] = AmplificationGap;
	XStop[2] = LXMeshHole + WidthMeshYlimb;
	YStop[2] = 0.0;
	ZStop[2] = AmplificationGap;
	NbSegments[2] = 1000;
	}

// Points - can be multiple points
NbPoints = 0;
if(NbPoints)
  {
  XPoint = dvector(1, NbPoints); YPoint = dvector(1, NbPoints);
  ZPoint = dvector(1, NbPoints);

  for(register int point = 1; point <= NbPoints; ++point)
    {
    XPoint[point] = 0.0; YPoint[point] = 0.0; ZPoint[point] = 0.0;
    }
  }

CreateMapFile(ModelInDir,
               NbMaps, NbBlock,
               NbXMap, NbYMap, NbZMap,
               MapLX, MapLY, MapLZ,
               MapXMid, MapYMid, MapZMid,
               NbLines, NbSegments,
               XStart, YStart, ZStart, XStop, YStop, ZStop,
               NbPoints, XPoint, YPoint, ZPoint);
}	// Map creation ends


// Complete the DeviceInputFile
int NbBCondns = 1;
EndDeviceInputFile(fModelInp, NbBCondns,
										DeviceOutDirName, ModelInDir, NbPrimitives, NbVolumes);

return 0;
}	// main ends
