// For neBEMV1.7.3
// Micromegas mesh being represented by wire, in contrast to surfaces that were
// used to represent the mesh surfaces in some of the other models.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>

#include <NR.h>
#include <Vector.h>
#include <Device.h>


int main(void)
{
#include "microMegas1.def"

// preparation of necessary input files
char DeviceInDir[256], DeviceInFileName[256], ModelSubDir[256], ModelInDir[256];
FILE *fModelInp;
strcpy(DeviceInDir, DeviceInDirName);	// existing directory
strcpy(ModelSubDir, ModelName);	// to be created
fModelInp = PrepareInputFiles(DeviceInDir, ModelSubDir,
															DeviceInFileName, ModelInDir);

// file-handler for device data file used for viewing
FILE *fData;
fData = PrepareViewingFiles(ModelInDir);

// Create volume reference data
// For this analysis, there are five volumes
// Anode pad conductor surface at the interface of conductor and gas volumes
// Anode pad substrate surface at the interface of dielectric and gas volumes
// Cathode mesh surface at the interface of conductor and gas volumes 
// Drift cathode surface at the interface of conductor and gas volumes 
// void CreateVolume(char ModelInDir[], int VolRef,
//                   int VolShape, int VolMaterial, int VolBoundaryType,
//                   double VolEpsilon, double VolPotential, double VolCharge);
int NbVolumes = 0;

{
// Gas: VolRef1
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 0, 0, 4, 1.0, 0.0, 0.0);

// Anode conductor: VolRef2
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, 0.0, 0.0);

// Dielectric: VolRef3
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 11, 4, 5.0, 0.0, 0.0);

// Cathode mesh conductor (wire): VolRef4
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 2, 2, 1, 0.0, -400.0, 0.0);

// Drift surface conductor: VolRef5
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, -3800.0, 0.0);
}	// Creation of volume reference data files over.


// Start building the device using primitives 
int NbPrimitives = 0;

// Anode - a conducting pad and two substrate strip limbs: rectangular shapes
{

// AnodeConductingPad
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 2; VolRef2 = 1; NbXSegs = 11; NbYSegs = 11;
SurfX = XAnodePad; SurfY = YAnodePad; SurfZ = ZAnodePad;
LengthX = LXAnodePad; LengthY = LYAnodePad;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal direction
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// AnodeConductingPad ends

// AnodeStripXlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 3; VolRef2 = 1; NbXSegs = 11; NbYSegs = 3;
SurfX = XStripXlimb; SurfY = YStripXlimb; SurfZ = ZStripXlimb;
LengthX = LXStripXlimb; LengthY = LYStripXlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// AnodeStripXlimb ends

// AnodeStripYlimb
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 3; VolRef2 = 1; NbXSegs = 3; NbYSegs = 11;
SurfX = XStripYlimb; SurfY = YStripYlimb; SurfZ = ZStripYlimb;
LengthX = LXStripYlimb; LengthY = LYStripYlimb;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// AnodeStripYlimb ends

} // Anode ends


// Cathode mesh made of two thin wires
{

// MeshXlimb
{
int nvertex;
int VolRef1, VolRef2, NbSegs;
double radius;	// radius
double xvert[2], yvert[2], zvert[2];
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

// dielectric surrounding the wire - can give rise
// to non-physical scenario: there is no interface between vacuum
// and these dielctric materials - this should be vacuum in order
// to avoid a new dielectric-dielectric interface
nvertex = 2; NbSegs = 11; VolRef1 = 4; VolRef2 = 1;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// vertices
xvert[0] = X0MeshXlimb; yvert[0] = Y0MeshXlimb; zvert[0] = Z0MeshXlimb;
xvert[1] = X1MeshXlimb; yvert[1] = Y1MeshXlimb; zvert[1] = Z1MeshXlimb;
radius = DiaMeshXlimb / 2.0;

CreateLine(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							radius, VolRef1, VolRef2, NbSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// MeshXlimb ends

// MeshYlimb
{
int nvertex;
int VolRef1, VolRef2, NbSegs;
double radius;	// radius
double xvert[2], yvert[2], zvert[2];
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

// dielectric surrounding the wire - can give rise
// to non-physical scenario: there is no interface between vacuum
// and these dielctric materials - this should be vacuum in order
// to avoid a new dielectric-dielectric interface
nvertex = 2; NbSegs = 11; VolRef1 = 4; VolRef2 = 1;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// vertices
xvert[0] = X0MeshYlimb; yvert[0] = Y0MeshYlimb; zvert[0] = Z0MeshYlimb;
xvert[1] = X1MeshYlimb; yvert[1] = Y1MeshYlimb; zvert[1] = Z1MeshYlimb;
radius = DiaMeshYlimb / 2.0;

CreateLine(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							radius, VolRef1, VolRef2, NbSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// MeshYlimb ends

} // Cathode mesh ends

// Drift plane - a conductor
{

// Drift plane
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
double SurfX, SurfY, SurfZ;	// CG of the rectangle
double LengthX, LengthY;	// length and breadth
int PeriodicTypeX, PeriodicTypeY, PeriodicTypeZ;
int PeriodicInX, PeriodicInY, PeriodicInZ;	// preiodicity info
double XPeriod, YPeriod, ZPeriod;

++NbPrimitives;

nvertex = 4; VolRef1 = 5; VolRef2 = 1; NbXSegs = 11; NbYSegs = 11;
SurfX = XDriftPlane; SurfY = YDriftPlane; SurfZ = ZDriftPlane;
LengthX = LXDriftPlane; LengthY = LYDriftPlane;
PeriodicTypeX = TypeXRepeat; PeriodicTypeY = TypeYRepeat; PeriodicTypeZ = TypeZRepeat;
PeriodicInX = NbXRepeat; PeriodicInY = NbYRepeat; PeriodicInZ = NbZRepeat;
XPeriod = LXRepeat; YPeriod = LYRepeat; ZPeriod = LZRepeat;

// compute vertices and specify normal
xvert[0] = SurfX-0.5*LengthX; yvert[0] = SurfY-0.5*LengthY; zvert[0] = SurfZ;
xvert[1] = SurfX-0.5*LengthX; yvert[1] = SurfY+0.5*LengthY; zvert[1] = SurfZ;
xvert[2] = SurfX+0.5*LengthX; yvert[2] = SurfY+0.5*LengthY; zvert[2] = SurfZ;
xvert[3] = SurfX+0.5*LengthX; yvert[3] = SurfY-0.5*LengthY; zvert[3] = SurfZ;
xnorm = 0.0; ynorm = 0.0; znorm = -1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// Drift plane ends

}	// Drift

// Close device data file
fclose(fData);


// Creation of map file to evaluate properties on maps (2d grids), along lines
// and at given points
{
// data that goes into the Map files
int NbMaps, NbLines, NbPoints;
int *NbXMap, *NbYMap, *NbZMap;
double *MapLX, *MapLY, *MapLZ;
double *MapXMid, *MapYMid, *MapZMid;
int *NbBlock;
int *NbSegments;
double *XStart, *YStart, *ZStart, *XStop, *YStop, *ZStop;
double *XPoint, *YPoint, *ZPoint;

// Maps - there can be multiple maps
NbMaps = 0;
if(NbMaps)
	{
	NbXMap = ivector(1, NbMaps); NbYMap = ivector(1, NbMaps);
	NbZMap = ivector(1, NbMaps); MapLX = dvector(1, NbMaps);
	MapLY = dvector(1, NbMaps); MapLZ = dvector(1, NbMaps);
	MapXMid = dvector(1, NbMaps); MapYMid = dvector(1, NbMaps);
	MapZMid = dvector(1, NbMaps); NbBlock = ivector(1, NbMaps);

	// Map 1
	NbXMap[1] = 100; NbYMap[1] = 100; NbZMap[1] = 1; // points along each axis
	MapLX[1] = LXMeshHole + 2.0*DiaMeshXlimb;
	MapLY[1] = LYMeshHole + 2.0*DiaMeshYlimb;
	MapLZ[1] = 0.0;	// lengths of the map
	MapXMid[1] = 0.0; MapYMid[1] = 0.0;
	MapZMid[1] = 0.5*AmplificationGap;	// mid-points
	NbBlock[1] = NbYMap[1];	// length of each block (in relation to gnuplot)
	}

// Lines - can be multiple lines
NbLines = 2;
if(NbLines)
	{
	NbSegments = ivector(1, NbLines); XStart = dvector(1, NbLines);
	YStart = dvector(1, NbLines); ZStart = dvector(1, NbLines);
	XStop = dvector(1, NbLines); YStop = dvector(1, NbLines);
	ZStop = dvector(1, NbLines);

	// Line 1
	XStart[1] = 0.0; YStart[1] = 0.0; ZStart[1] = 0.0;
	XStop[1] = 0.0; YStop[1] = 0.0; ZStop[1] = ZDriftPlane;
	NbSegments[1] = 1000;

	// Line 2
	XStart[2] = -(LXMeshHole + DiaMeshXlimb);
	YStart[2] = 0.0;
	ZStart[2] = AmplificationGap + 0.5*DiaMeshXlimb;
	XStop[2] = LXMeshHole + DiaMeshXlimb;
	YStop[2] = 0.0;
	ZStop[2] = AmplificationGap + 0.5*DiaMeshXlimb;
	NbSegments[2] = 1000;
	}

// Points - can be multiple points
NbPoints = 0;
if(NbPoints)
  {
  XPoint = dvector(1, NbPoints); YPoint = dvector(1, NbPoints);
  ZPoint = dvector(1, NbPoints);

  for(register int point = 1; point <= NbPoints; ++point)
    {
    XPoint[point] = 0.0; YPoint[point] = 0.0; ZPoint[point] = 0.0;
    }
  }

CreateMapFile(ModelInDir,
               NbMaps, NbBlock,
               NbXMap, NbYMap, NbZMap,
               MapLX, MapLY, MapLZ,
               MapXMid, MapYMid, MapZMid,
               NbLines, NbSegments,
               XStart, YStart, ZStart, XStop, YStop, ZStop,
               NbPoints, XPoint, YPoint, ZPoint);
}	// Map creation ends


// Complete the DeviceInputFile
int NbBCondns = 1;
EndDeviceInputFile(fModelInp, NbBCondns,
										DeviceOutDirName, ModelInDir, NbPrimitives, NbVolumes);

return 0;
}	// main ends
