/*
(c) 2009 Supratik Mukhopadhyay, Nayana Majumdar
*/
#include <stdio.h>
#include <assert.h>
#include <sys/stat.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>


// Assign default values to some of the important global variables
int neBEMSetDefaults(void)
{
neBEMState = 0;

NbVolumes = 2;
VolMax = 2;

NbPrimitives = 1;
MaxNbVertices = 4;

NbSurfs = 1;
NbWires = 0;

MinNbElementsOnLength = 1;
MaxNbElementsOnLength = 100;
ElementLengthRqstd = 2.0e-6;

NewModel = 1; NewMesh = 1; NewBC = 1; NewPP = 1;
ModelCntr = 1; MeshCntr = 1; BCCntr = 1; PPCntr = 1;

DebugLevel = 0;

LengthScale = 1.0;

TimeStep = 1;

char *homedir;
homedir = getenv("HOME");
if(homedir != NULL)
  {printf("user homedir is %s\n", homedir);}
else
  {
  printf("problem determining home directory! ... returning ...\n");
  return -1;
  }

strcpy(DeviceOutDir, homedir);
strcat(DeviceOutDir, "/neBEMWD/Outputs/DI/V");
strcat(DeviceOutDir, neBEMVersion);
strcat(DeviceOutDir, "/");

OptDeviceFile = 0;
strcpy(DeviceInputFile, "");

OptPrintPrimaryDetails = 0;
OptPrintVolumeDetails = 0;
OptPrintVertexAndNormal = 0;

OptGnuplot = 0; OptGnuplotPrimitives = 0; OptGnuplotElements = 0;
OptPrimitiveFiles = 0; OptElementFiles = 0;

OptReuseDir = 1;  // Same directory can be used over and over again - BEWARE!

OptInvMatProc = 0;  // matrix inversion procedure => 0: LU, 1: SVD

OptValidateSolution = 0;  // No validation
OptForceValidation = 0;  // No validation
OptStorePrimitives = 1;
OptStoreElements = 1;
OptStoreInflMatrix = 0; // not storing influence matrix
OptStoreInvMatrix = 0;  // not storing inverted matrix
OptFormattedFile = 0; // stored matrix not a formatted file
OptUnformattedFile = 0; // stored matrix not an unformatted file
OptRepeatLHMatrix = 0;  // do not repeat the LH matrix for validation

OptSystemChargeZero = 0;	// do not enforce total system charge equal to zero

// The above parameters can be overridden by an `init' file.
char initFileName[256];
strcpy(initFileName, homedir);
strcat(initFileName, "/neBEMWD/InitFiles/");
strcat(initFileName, "DI2neBEM.init");

struct stat st;
if(stat(initFileName, &st) == 0)
	{
	printf("Found an init file ... updating parameters.\n");
	int fstatus = ReadInitFile(initFileName);
	assert(fstatus == 0);
	}
else
	{
	printf("No init file ... continuing with the parameter values.\n");
	}

return(0);
} // end of neBEMSetDefaults


int ReadInitFile(char filename[])
{
FILE *finit;
finit = fopen(filename, "r");
assert(finit != NULL);

fscanf(finit, "MinNbElementsOnLength: %d\n", &MinNbElementsOnLength);
fscanf(finit, "MaxNbElementsOnLength: %d\n", &MaxNbElementsOnLength);
fscanf(finit, "ElementLengthRqstd: %le\n", &ElementLengthRqstd);
fscanf(finit, "LengthScale: %le\n", &LengthScale);

fscanf(finit, "DebugLevel: %d\n", &DebugLevel);

fscanf(finit, "NewModel: %d\n", &NewModel);
fscanf(finit, "NewMesh: %d\n", &NewMesh);
fscanf(finit, "NewBC: %d\n", &NewBC);
fscanf(finit, "NewPP: %d\n", &NewPP);
fscanf(finit, "ModelCntr: %d\n", &ModelCntr);
fscanf(finit, "MeshCntr: %d\n", &MeshCntr);
fscanf(finit, "BCCntr: %d\n", &BCCntr);
fscanf(finit, "PPCntr: %d\n", &PPCntr);

fscanf(finit, "DeviceOutDir: %s\n", DeviceOutDir);

fscanf(finit, "OptDeviceFile: %d\n", &OptDeviceFile);
fscanf(finit, "DeviceInputFile: %s\n", DeviceInputFile);

fscanf(finit, "OptPrintPrimaryDetails: %d\n", &OptPrintPrimaryDetails);
fscanf(finit, "OptPrintVolumeDetails: %d\n", &OptPrintVolumeDetails);
fscanf(finit, "OptPrintVertexAndNormal: %d\n", &OptPrintVertexAndNormal);

fscanf(finit, "OptGnuplot: %d\n", &OptGnuplot);
fscanf(finit, "OptGnuplotPrimitives: %d\n", &OptGnuplotPrimitives);
fscanf(finit, "OptGnuplotElements: %d\n", &OptGnuplotElements);

fscanf(finit, "OptPrimitiveFiles: %d\n", &OptPrimitiveFiles);
fscanf(finit, "OptElementFiles: %d\n", &OptElementFiles);

fscanf(finit, "OptReuseDir: %d\n", &OptReuseDir);

fscanf(finit, "OptInvMatProc: %d\n", &OptInvMatProc);

fscanf(finit, "OptValidateSolution: %d\n", &OptValidateSolution);
fscanf(finit, "OptForceValidation: %d\n", &OptForceValidation);
fscanf(finit, "OptStorePrimitives: %d\n", &OptStorePrimitives);
fscanf(finit, "OptStoreElements: %d\n", &OptStoreElements);
fscanf(finit, "OptStoreInflMatrix: %d\n", &OptStoreInflMatrix);
fscanf(finit, "OptStoreInvMatrix: %d\n", &OptStoreInvMatrix);
fscanf(finit, "OptFormattedFile: %d\n", &OptFormattedFile);
fscanf(finit, "OptUnformattedFile: %d\n", &OptUnformattedFile);
fscanf(finit, "OptRepeatLHMatrix: %d\n", &OptRepeatLHMatrix);

fscanf(finit, "OptSystemChargeZero: %d\n", &OptSystemChargeZero);

fclose(finit);

return 0;
}


// No input from files
int neBEMGetInputsFromFiles(void)
{
return(0);
} // end of neBEMGetInputsFromFiles


int neBEMGetNbPrimitives(void)
{
return 5;
}	// neBEMGetNbPrimitives end here


// Modified distances and potential (08/11/11)
// Unmodifed block maintained below.
int neBEMGetPrimitive(int prim, int *nvertex,
                      double xvert[], double yvert[], double zvert[],
                      double *xnorm, double *ynorm, double *znorm,
                      int *volref1, int *volref2)
{
/*
switch(prim)	// No degenerate surface
	{
	case 1:		// plate_top
		*nvertex = 4;
		xvert[0] = -10.0e-2; yvert[0] = 10.0e-2; zvert[0] = -10.0e-2;
		xvert[1] = +10.0e-2; yvert[1] = 10.0e-2; zvert[1] = -10.0e-2;
		xvert[2] = +10.0e-2; yvert[2] = 10.0e-2; zvert[2] = +10.0e-2;
		xvert[3] = -10.0e-2; yvert[3] = 10.0e-2; zvert[3] = +10.0e-2;
		*xnorm = 0.0; *ynorm = -1.0; *znorm = 0.0;
		*volref1 = 1; // self
		*volref2 = 0; // 
		break;
	case 2:		// plate_bottom
		*nvertex = 4;
		xvert[0] = -10.0e-2; yvert[0] = -10.0e-2; zvert[0] = -10.0e-2;
		xvert[1] = +10.0e-2; yvert[1] = -10.0e-2; zvert[1] = -10.0e-2;
		xvert[2] = +10.0e-2; yvert[2] = -10.0e-2; zvert[2] = +10.0e-2;
		xvert[3] = -10.0e-2; yvert[3] = -10.0e-2; zvert[3] = +10.0e-2;
		*xnorm = 0.0; *ynorm = 1.0; *znorm = 0.0;
		*volref1 = 2; // self
		*volref2 = 0; // 
		break;
	case 3:		// plate_left
		*nvertex = 4;
		xvert[0] = -10.0e-2; yvert[0] = -10.0e-2; zvert[0] = -10.0e-2;
		xvert[1] = -10.0e-2; yvert[1] = 10.0e-2; zvert[1] = -10.0e-2;
		xvert[2] = -10.0e-2; yvert[2] = 10.0e-2; zvert[2] = +10.0e-2;
		xvert[3] = -10.0e-2; yvert[3] = -10.0e-2; zvert[3] = +10.0e-2;
		*xnorm = 1.0; *ynorm = 0.0; *znorm = 0.0;
		*volref1 = 3; // self
		*volref2 = 0; // 
		break;
	case 4:		// plate_right
		*nvertex = 4;
		xvert[0] = +10.0e-2; yvert[0] = -10.0e-2; zvert[0] = -10.0e-2;
		xvert[1] = +10.0e-2; yvert[1] = 10.0e-2; zvert[1] = -10.0e-2;
		xvert[2] = +10.0e-2; yvert[2] = 10.0e-2; zvert[2] = +10.0e-2;
		xvert[3] = +10.0e-2; yvert[3] = -10.0e-2; zvert[3] = +10.0e-2;
		*xnorm = -1.0; *ynorm = 0.0; *znorm = 0.0;
		*volref1 = 4; // self
		*volref2 = 0; //
		break;
	case 5:		// plate_mid
		*nvertex = 4;
		xvert[0] = -10.0e-2; yvert[0] = 0.0e-2; zvert[0] = -10.0e-2;
		xvert[1] = +10.0e-2; yvert[1] = 0.0e-2; zvert[1] = -10.0e-2;
		xvert[2] = +10.0e-2; yvert[2] = 0.0e-2; zvert[2] = +10.0e-2;
		xvert[3] = -10.0e-2; yvert[3] = 0.0e-2; zvert[3] = +10.0e-2;
		*xnorm = 0.0; *ynorm = 1.0; *znorm = 0.0;
		*volref1 = 5; // self
		*volref2 = 0; //
		break;
	default:
		*nvertex = 0;
		return -1;
		break;
	}	// switch prim
*/

/* */
switch(prim)	// No degenerate surface
	{
	case 1:		// plate_top
		*nvertex = 4;
		xvert[0] = -0.0; yvert[0] = 20.0e-6; zvert[0] = -50.0e-6;
		xvert[1] = +30.0e-6; yvert[1] = 20.0e-6; zvert[1] = -50.0e-6;
		xvert[2] = +30.0e-6; yvert[2] = 20.0e-6; zvert[2] = +50.0e-6;
		xvert[3] = -0.0; yvert[3] = 20.0e-6; zvert[3] = +50.0e-6;
		*xnorm = 0.0; *ynorm = 1.0; *znorm = 0.0;
		*volref1 = 1; // self
		*volref2 = 0; // 
		break;
	case 2:		// plate_bottom
		*nvertex = 4;
		xvert[0] = -0.0; yvert[0] = 0.0; zvert[0] = -50.0e-6;
		xvert[1] = +30.0e-6; yvert[1] = 0.0; zvert[1] = -50.0e-6;
		xvert[2] = +30.0e-6; yvert[2] = 0.0; zvert[2] = +50.0e-6;
		xvert[3] = -0.0; yvert[3] = 0.0; zvert[3] = +50.0e-6;
		*xnorm = 0.0; *ynorm = -1.0; *znorm = 0.0;
		*volref1 = 2; // self
		*volref2 = 0; // 
		break;
	case 3:		// plate_left
		*nvertex = 4;
		xvert[0] = -0.0; yvert[0] = 0.0e-6; zvert[0] = -50.0e-6;
		xvert[1] = +0.0; yvert[1] = 20.0e-6; zvert[1] = -50.0e-6;
		xvert[2] = +0.0; yvert[2] = 20.0e-6; zvert[2] = +50.0e-6;
		xvert[3] = -0.0; yvert[3] = 0.0e-6; zvert[3] = +50.0e-6;
		*xnorm = -1.0; *ynorm = 0.0; *znorm = 0.0;
		*volref1 = 3; // self
		*volref2 = 0; // 
		break;
	case 4:		// plate_right
		*nvertex = 4;
		xvert[0] = +30.0e-6; yvert[0] = 0.0e-6; zvert[0] = -50.0e-6;
		xvert[1] = +30.0e-6; yvert[1] = 20.0e-6; zvert[1] = -50.0e-6;
		xvert[2] = +30.0e-6; yvert[2] = 20.0e-6; zvert[2] = +50.0e-6;
		xvert[3] = +30.0e-6; yvert[3] = 0.0e-6; zvert[3] = +50.0e-6;
		*xnorm = 1.0; *ynorm = 0.0; *znorm = 0.0;
		*volref1 = 4; // self
		*volref2 = 0; //
		break;
	case 5:		// plate_mid
		*nvertex = 4;
		xvert[0] = -0.0; yvert[0] = 10.0e-6; zvert[0] = -50.0e-6;
		xvert[1] = +30.0e-6; yvert[1] = 10.0e-6; zvert[1] = -50.0e-6;
		xvert[2] = +30.0e-6; yvert[2] = 10.0e-6; zvert[2] = +50.0e-6;
		xvert[3] = -0.0; yvert[3] = 10.0e-6; zvert[3] = +50.0e-6;
		*xnorm = 0.0; *ynorm = 1.0; *znorm = 0.0;
		*volref1 = 5; // self
		*volref2 = 0; //
		break;
	default:
		*nvertex = 0;
		return -1;
		break;
	}	// switch prim
/* */

return 0;
}	// neBEMGetPrimitive ends here


int neBEMVolumeDescription(int volref, int *shape,
                           int *material, double *epsilon,
                           double *potential, double *charge,
                           int *boundarytype)
{
switch(volref)
	{
	case 0:	// Encompassing volume
		*shape = 0;	// extends to infinity
		*material = 0;	// vacuum
		*epsilon = 1.0;	// none
		*potential = 0.0;	// none
		*charge = 0.0;		// none
		*boundarytype = 4;	// no boundary
		break;

	case 1:
		*shape = 4;	// quadrilateral
		*material = 1;	// conductor
		*epsilon = 0.0;	// none
		// *potential = 1.0e+2;	// supplied (raised by e+2)
		*potential = 1.0;	// supplied
		*charge = 0.0;		// unknown
		*boundarytype = 1;	// conductor with known potential
		break;

	case 2:
		*shape = 4;	// quadrilateral
		*material = 1;	// conductor
		*epsilon = 0.0;	// none
		*potential = 0.0;	// supplied
		*charge = 0.0;		// none
		*boundarytype = 1;	// conductor with known potential
		break;

	case 3:
		*shape = 4;	// quadrilateral
		*material = 1;	// conductor
		*epsilon = 0.0;	// none
		*potential = 0.0;	// supplied
		*charge = 0.0;		// none
		*boundarytype = 1;	// conductor with known potential
		break;

	case 4:
		*shape = 4;	// quadrilateral
		*material = 1;	// conductor
		*epsilon = 0.0;	// none
		*potential = 0.0;	// supplied
		*charge = 0.0;		// none
		*boundarytype = 1;	// conductor with known potential
		break;

	case 5:
		*shape = 4;	// quadrilateral
		*material = 11;	// dielectric
		*epsilon = 10.0;	// 10.0 and 0.1 takes care of the two cases
                    // epsilon here is epsilon2 in the ref. So, epsilon=10
                    // implies R = 0.1, and epsilon=0.1 implies R=10
		*potential = 0.0;	// none
		*charge = 0.0;		// none
		*boundarytype = 4;	// dielectric-dielectric boundary
		break;

	default:	// Default is vacuum, but report out of range
		*shape = 0;	// extends to infinity
		*material = 0;	// vacuum
		*epsilon = 0.0;	// none
		*potential = 0.0;	// none
		*charge = 0.0;		// none
		*boundarytype = 0;	// no boundary
		printf("WARNING: volume reference (volref) %d out of range ...\n",
				volref);
		return -1;
		break;
	}	// switch on volref ends

return 0;
}	// neBEMVolumeDescription ends here


int neBEMVolumePoint(double x, double y, double z)
{
if( (fabs(x) <= 0.5)
	&& (fabs(y) <= 1.0e-12)
	&& (fabs(z) <= 0.5) )
	return 1;	// on the plate
else
	return(0);	// in the encompassing volume
}	// neBEMVolumePoint ends here


// Reads in periodicity information.
// ix: PeriodicTypeX (1 - simple, 2 - mirror, 3 - axial, 4 - rotation)
// jx: PeriodicInX (Number of copies internal to neBEM)
// sx: XPeriod
int neBEMGetPeriodicities(int prim, int *ix, int *jx, double *sx,
																		int *iy, int *jy, double *sy,
                            				int *iz, int *jz, double *sz)
{
*ix = 0; *jx = 0; *sx = 1.0;
*iy = 0; *jy = 0; *sy = 1.0;
*iz = 0; *jz = 0; *sz = 1.0;

return 0;
}	// neBEMGetPeriodicities end


// Information on bounding planes
// ixmin=0: lower x-plane does not exist
// ixmin=1: lower x-plane does exist
// cxmin: coordinate of lower x-plane
// vxmin: potential of lower x-plane
// Similar for ixmax, iymin, iymax, izmin, izmax
int neBEMGetBoundingPlanes(int *ixmin, double *cxmin, double *vxmin,
                            int *ixmax, double *cxmax, double *vxmax,
                            int *iymin, double *cymin, double *vymin,
                            int *iymax, double *cymax, double *vymax,
                            int *izmin, double *czmin, double *vzmin,
                            int *izmax, double *czmax, double *vzmax)
{
*ixmin = 0; *ixmax = 0;
*iymin = 0; *iymax = 0;
*izmin = 0; *izmax = 0;
*cxmin = 0.0; *cxmax = 0.0;
*cymin = 0.0; *cymax = 0.0;
*czmin = 0.0; *czmax = 0.0;
*vxmin = 0.0; *vxmax = 0.0;
*vymin = 0.0; *vymax = 0.0;
*vzmin = 0.0; *vzmax = 0.0;

return 0;
} // neBEMGetBoundingPlanes ends


int neBEMGetMirror(int prim,
        int* ix, int* jx, double* sx,
        int* iy, int* jy, double* sy,
        int* iz, int* jz, double* sz)
{
*ix = *iy = *iz = 0;  // no mirror now
*jx = *jy = *jz = 0;    // not used at present
*sx = *sy = *sz = 0.0;  // mirror assumed to be passing through the origin

// Only value 2 implies mirror periodicity
// At present MirrorTypeX etc is always 2. When Mirrors of other kinds will be
// allowed, this will need modification.
if((*ix != 2) && (*iy != 2) && (*iz != 2))
  {
  *ix = *iy = *iz = 0;
  }

// Only one reflection is allowed at present
if(*ix == 2)
  { *iy = *iz = 0;}
else if (*iy == 2)
  *iz = 0;

// Return error status.
return 0;
} // end of neBEMGetMirror
