/*
(c) 2009 Supratik Mukhopadhyay, Nayana Majumdar
*/
// This code can work as a template interface for codes calling neBEM toolkit
// In the interface, the user has to supply
// int neBEMGetNbPrimitives(void)
// int neBEMGetPrimitive(int prim, int *nvertex,
//                       double xvert[], double yvert[], double zvert[],
//                       double *xnorm, double *ynorm, double *znorm,
//                       int *volref1, int *volref2)
// int neBEMVolumeDescription(int volref1, int *shape1,
//                            int *material1, double *epsilon1,
//                            double *potential1, double *charge1,
//                            int *boundarytype1)
// int neBEMVolumePoint(double x, double y, double z)
// int neBEMGetPeriodicities(int *ix, double *sx, int *iy, double *sy,
//                             int *iz, double *sz)

// Gdp electrostatics using neBEM
// Inputs read in from files that are created separately using preprocessor
// codes.
// Gets number of primitives (polygonal surfaces (2D) and wires (1D)) and
// their geometrical details
// Missing information are:
// No classification of primitives - we can have at least two, 2D and 1D
//  Soln: assumed only 2D surfaces; as a result, NbWires = 0
// No hint about the discretization requirements
// Assumptions:
//	MaxNbVertices: 4 (maximum nb of vertices a polygon can have)

#include <stdio.h>
#include <assert.h>
#include <sys/stat.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>


// Assign default values to some of the important global variables
int neBEMSetDefaults(void)
{
neBEMState = 0;

NbVolumes = 2;
VolMax = 2;

NbPrimitives = 1;
MaxNbVertices = 4;

NbSurfs = 1;
NbWires = 0;

MinNbElementsOnLength = 5;
MaxNbElementsOnLength = 51;
ElementLengthRqstd = 25.0e-3;

NewModel = 1; NewMesh = 1; NewBC = 1; NewPP = 1;
ModelCntr = 1; MeshCntr = 1; BCCntr = 1; PPCntr = 1;

DebugLevel = 0;

LengthScale = 1.0;

TimeStep = 1;

char *homedir;
homedir = getenv("HOME");
if(homedir != NULL)
  {printf("user homedir is %s\n", homedir);}
else
  {
  printf("problem determining home directory! ... returning ...\n");
  return -1;
  }

strcpy(DeviceOutDir, homedir);
strcat(DeviceOutDir, "/neBEMWD/Outputs/HalfPlateYMirror/V");
strcat(DeviceOutDir, neBEMVersion);
strcat(DeviceOutDir, "/");

OptDeviceFile = 1;
strcpy(DeviceInputFile, homedir);
strcat(DeviceInputFile, "/neBEMWD/Inputs/HalfPlateYMirror/Model1.inp");

OptPrintPrimaryDetails = 0;
OptPrintVolumeDetails = 0;
OptPrintVertexAndNormal = 0;

OptGnuplot = 0; OptGnuplotPrimitives = 0; OptGnuplotElements = 0;
OptPrimitiveFiles = 0; OptElementFiles = 0;

OptReuseDir = 1;  // Same directory can be used over and over again - BEWARE!

OptInvMatProc = 0;  // matrix inversion procedure => 0: LU, 1: SVD

OptValidateSolution = 1;  // No validation
OptForceValidation = 1;  // No validation
OptStorePrimitives = 1;
OptStoreElements = 1;
OptStoreInflMatrix = 0; // not storing influence matrix
OptStoreInvMatrix = 0;  // not storing inverted matrix
OptFormattedFile = 0; // stored matrix not a formatted file
OptUnformattedFile = 0; // stored matrix not an unformatted file
OptRepeatLHMatrix = 1;  // do not repeat the LH matrix for validation

OptSystemChargeZero = 0;	// Enforce total system charge to be zero

// The above parameters can be overridden by an `init' file.
char initFileName[256];
strcpy(initFileName, homedir);
strcat(initFileName, "/neBEMWD/InitFiles/");
strcat(initFileName, "HalfPlateYMirror2neBEM.init");

struct stat st;
if(stat(initFileName, &st) == 0)
	{
	printf("Found an init file ... updating parameters.\n");
	int fstatus = ReadInitFile(initFileName);
	assert(fstatus == 0);
	}
else
	{
	printf("No init file ... continuing with the parameter values.\n");
	}

return(0);
}	// end of neBEMSetDefaults


int ReadInitFile(char filename[])
{
FILE *finit;
finit = fopen(filename, "r");
assert(finit != NULL);

fscanf(finit, "MinNbElementsOnLength: %d\n", &MinNbElementsOnLength);
fscanf(finit, "MaxNbElementsOnLength: %d\n", &MaxNbElementsOnLength);
fscanf(finit, "ElementLengthRqstd: %le\n", &ElementLengthRqstd);
fscanf(finit, "LengthScale: %le\n", &LengthScale);

fscanf(finit, "DebugLevel: %d\n", &DebugLevel);

fscanf(finit, "NewModel: %d\n", &NewModel);
fscanf(finit, "NewMesh: %d\n", &NewMesh);
fscanf(finit, "NewBC: %d\n", &NewBC);
fscanf(finit, "NewPP: %d\n", &NewPP);
fscanf(finit, "ModelCntr: %d\n", &ModelCntr);
fscanf(finit, "MeshCntr: %d\n", &MeshCntr);
fscanf(finit, "BCCntr: %d\n", &BCCntr);
fscanf(finit, "PPCntr: %d\n", &PPCntr);

fscanf(finit, "DeviceOutDir: %s\n", DeviceOutDir);

fscanf(finit, "OptDeviceFile: %d\n", &OptDeviceFile);
fscanf(finit, "DeviceInputFile: %s\n", DeviceInputFile);

fscanf(finit, "OptPrintPrimaryDetails: %d\n", &OptPrintPrimaryDetails);
fscanf(finit, "OptPrintVolumeDetails: %d\n", &OptPrintVolumeDetails);
fscanf(finit, "OptPrintVertexAndNormal: %d\n", &OptPrintVertexAndNormal);

fscanf(finit, "OptGnuplot: %d\n", &OptGnuplot);
fscanf(finit, "OptGnuplotPrimitives: %d\n", &OptGnuplotPrimitives);
fscanf(finit, "OptGnuplotElements: %d\n", &OptGnuplotElements);

fscanf(finit, "OptPrimitiveFiles: %d\n", &OptPrimitiveFiles);
fscanf(finit, "OptElementFiles: %d\n", &OptElementFiles);

fscanf(finit, "OptReuseDir: %d\n", &OptReuseDir);

fscanf(finit, "OptInvMatProc: %d\n", &OptInvMatProc);

fscanf(finit, "OptValidateSolution: %d\n", &OptValidateSolution);
fscanf(finit, "OptForceValidation: %d\n", &OptForceValidation);
fscanf(finit, "OptStorePrimitives: %d\n", &OptStorePrimitives);
fscanf(finit, "OptStoreElements: %d\n", &OptStoreElements);
fscanf(finit, "OptStoreInflMatrix: %d\n", &OptStoreInflMatrix);
fscanf(finit, "OptStoreInvMatrix: %d\n", &OptStoreInvMatrix);
fscanf(finit, "OptFormattedFile: %d\n", &OptFormattedFile);
fscanf(finit, "OptUnformattedFile: %d\n", &OptUnformattedFile);
fscanf(finit, "OptRepeatLHMatrix: %d\n", &OptRepeatLHMatrix);

fscanf(finit, "OptSystemChargeZero: %d\n", &OptSystemChargeZero);

fclose(finit);

return 0;
}	// end of ReadInitFile


int neBEMGetInputsFromFiles(void)
{
char instr[256], InDir[256], tmpInFile[256], PrimInFile[256], VolInFile[256];
FILE *fin;
FILE *fPrim, *fVol;

// These are ad-hoc limits to the possible numbers of volumes and vertices
MaxNbVertices = 6;

fin = fopen(DeviceInputFile, "r");
assert(fin != NULL);

fgets(instr, 256, fin); fscanf(fin, "%s\n", InDir);
if(1){printf("#%s", instr); printf("%s\n", InDir);}

fgets(instr, 256, fin); fscanf(fin, "%d\n", &tmpNbPrimitives);
if(1){printf("#%s", instr); printf("%d\n", tmpNbPrimitives);}

fgets(instr, 256, fin); fscanf(fin, "%d\n", &NbVolumes);
if(1){printf("#%s", instr); printf("%d\n", NbVolumes);}

// allocate tmp variable spaces
tmpVolShape = ivector(1, NbVolumes);
tmpVolMaterial = ivector(1, NbVolumes);
tmpVolEpsilon = dvector(1, NbVolumes);
tmpVolPotential = dvector(1, NbVolumes);
tmpVolCharge = dvector(1, NbVolumes);
tmpVolBoundaryType = ivector(1, NbVolumes);
tmpNbVertices = ivector(1, tmpNbPrimitives);
tmpVolRef1 = ivector(1, tmpNbPrimitives);
tmpVolRef2 = ivector(1, tmpNbPrimitives);
tmpNbXSegs = ivector(1, tmpNbPrimitives);
tmpNbZSegs = ivector(1, tmpNbPrimitives);
tmpXvert = dmatrix(1, tmpNbPrimitives, 1, MaxNbVertices);
tmpYvert = dmatrix(1, tmpNbPrimitives, 1, MaxNbVertices);
tmpZvert = dmatrix(1, tmpNbPrimitives, 1, MaxNbVertices);
tmpXNorm = dvector(1, tmpNbPrimitives);
tmpYNorm = dvector(1, tmpNbPrimitives);
tmpZNorm = dvector(1, tmpNbPrimitives);
tmpPeriodicTypeX = ivector(1, tmpNbPrimitives);
tmpPeriodicInX = ivector(1, tmpNbPrimitives);
tmpXPeriod = dvector(1, tmpNbPrimitives);
tmpPeriodicTypeY = ivector(1, tmpNbPrimitives);
tmpPeriodicInY = ivector(1, tmpNbPrimitives);
tmpYPeriod = dvector(1, tmpNbPrimitives);
tmpPeriodicTypeZ = ivector(1, tmpNbPrimitives);
tmpPeriodicInZ = ivector(1, tmpNbPrimitives);
tmpZPeriod = dvector(1, tmpNbPrimitives);

for(register int prim = 1; prim <= tmpNbPrimitives; ++prim)
  {
	if(0)
		printf("Details for primitive %d\n", prim);
  fgets(instr, 256, fin); fscanf(fin, "%s\n", tmpInFile);
	strcpy(PrimInFile, InDir); strcat(PrimInFile, tmpInFile);
	if(0)
		printf("Primitive file: %s\n", PrimInFile);

	fPrim = fopen(PrimInFile, "r");
	assert(fPrim != NULL);

	fgets(instr, 256, fPrim); fscanf(fPrim, "%d\n", &tmpNbVertices[prim]);
	if(0)
		{printf("#%s", instr); printf("%d\n", tmpNbVertices[prim]);}
	if(tmpNbVertices[prim] > MaxNbVertices)
		{
		printf("Nb of vertices crosses limit for primitive %d\n", prim);
		printf("Cannot proceed ... aborting.\n");
		return(-1);
		}

	for(register int vert = 0; vert < tmpNbVertices[prim]; ++vert)
		{
		fgets(instr, 256, fPrim);
		fscanf(fPrim, "%le %le %le\n",
					&tmpXvert[prim][vert], &tmpYvert[prim][vert], &tmpZvert[prim][vert]);
		if(0)
			printf("%s %lg, %lg, %lg\n", instr,
						tmpXvert[prim][vert], tmpYvert[prim][vert], tmpZvert[prim][vert]);
		}

	fgets(instr, 256, fPrim);
	if(tmpNbVertices[prim] == 2)	// a wire
		{
		fscanf(fPrim, "%le\n", &tmpXNorm[prim]);
		tmpYNorm[prim] = 0.0; tmpZNorm[prim] = 0.0;	// dummies
		if(0)
			printf("%s %lg\n", instr, tmpXNorm[prim]);
		}
	else										// a surface - look-out for other possibilities!
		{
		fscanf(fPrim, "%le %le %le\n",
										&tmpXNorm[prim], &tmpYNorm[prim], &tmpZNorm[prim]);
		if(0)
			printf("%s %lg, %lg, %lg\n", instr,
							tmpXNorm[prim], tmpYNorm[prim], tmpZNorm[prim]);
		}

	fgets(instr, 256, fPrim);
	fscanf(fPrim, "%d %d\n", &tmpVolRef1[prim], &tmpVolRef2[prim]);
	if(0)
		printf("%s %d, %d\n", instr, tmpVolRef1[prim], tmpVolRef1[prim]);
	if( (tmpVolRef1[prim] > NbVolumes)
			|| (tmpVolRef2[prim] > NbVolumes) )
		{
		printf("Nb of volumes crosses limit for primitive %d\n", prim);
		printf("Cannot proceed ... aborting.\n");
		return(-1);
		}
	
	fgets(instr, 256, fPrim);
	if(tmpNbVertices[prim] == 2)	// a wire
		{
		fscanf(fPrim, "%d\n", &tmpNbXSegs[prim]);
		tmpNbZSegs[prim] = 0;	// dummy
		if(0)
			printf("%s %d\n", instr, tmpNbXSegs[prim]);
		}
	else										// a surface - look-out for other possibilities!
		{
		fscanf(fPrim, "%d %d\n", &tmpNbXSegs[prim], &tmpNbZSegs[prim]);
		if(0)
			printf("%s %d, %d\n", instr, tmpNbXSegs[prim], tmpNbZSegs[prim]);
		}

  fgets(instr, 256, fPrim);
  fscanf(fPrim, "%d %d %le\n",
            &tmpPeriodicTypeX[prim], &tmpPeriodicInX[prim], &tmpXPeriod[prim]);
  if(0)
    printf("%s %d, %d, %le\n",
        instr, tmpPeriodicTypeX[prim], tmpPeriodicInX[prim], tmpXPeriod[prim]);

  fgets(instr, 256, fPrim);
  fscanf(fPrim, "%d %d %le\n",
            &tmpPeriodicTypeY[prim], &tmpPeriodicInY[prim], &tmpYPeriod[prim]);
  if(0)
    printf("%s %d, %d, %le\n",
        instr, tmpPeriodicTypeY[prim], tmpPeriodicInY[prim], tmpYPeriod[prim]);

  fgets(instr, 256, fPrim);
  fscanf(fPrim, "%d %d %le\n",
            &tmpPeriodicTypeZ[prim], &tmpPeriodicInZ[prim], &tmpZPeriod[prim]);
  if(0)
    printf("%s %d, %d, %le\n",
        instr, tmpPeriodicTypeZ[prim], tmpPeriodicInZ[prim], tmpZPeriod[prim]);

	fclose(fPrim);
	if(0) getchar();
  }	// loop over all the primitives for getting inputs, over

int NoBC;	// this info has to finally go into the element structure
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NoBC);	// dummy, at present
if(1) {printf("%s %d\n", instr, NoBC); fflush(stdout);}

strcpy(GnuplotTmpDir, "GViewDir");
strcpy(GnuplotScriptFile, "GViewDir/gviewer.gp");

fgets(instr, 256, fin); fscanf(fin, "%s\n", DeviceOutDir);
if(1) {printf("#%s", instr); printf("%s\n", DeviceOutDir);}

char infile[256];
fgets(instr, 256, fin); fscanf(fin, "%s\n", infile);
strcpy(MapFile, InDir); strcat(MapFile, infile);
if(1) {printf("#%s", instr); printf("%s\n", MapFile);}

fclose(fin);

char strvol[3];
for(register int vol = 1; vol <= NbVolumes; ++vol)
	{
	sprintf(strvol, "%d", vol);
	strcpy(VolInFile, InDir); strcat(VolInFile, "Volume");
	strcat(VolInFile, strvol); strcat(VolInFile, ".inp");

	fVol = fopen(VolInFile, "r");
	assert(fVol != NULL);
	
	fgets(instr, 256, fin); fscanf(fVol, "%d\n", &tmpVolShape[vol]);
	fgets(instr, 256, fin); fscanf(fVol, "%d\n", &tmpVolMaterial[vol]);
	fgets(instr, 256, fin); fscanf(fVol, "%le\n", &tmpVolEpsilon[vol]);
	fgets(instr, 256, fin); fscanf(fVol, "%le\n", &tmpVolPotential[vol]);
	fgets(instr, 256, fin); fscanf(fVol, "%le\n", &tmpVolCharge[vol]);
	fgets(instr, 256, fin); fscanf(fVol, "%d\n", &tmpVolBoundaryType[vol]);

	fclose(fVol);
	}

return(0);
}


int neBEMGetNbPrimitives(void)
{
return(tmpNbPrimitives);
}	// neBEMGetNbPrimitives end here


int neBEMGetPrimitive(int prim, int *nvertex,
                      double xvert[], double yvert[], double zvert[],
                      double *xnorm, double *ynorm, double *znorm,
                      int *volref1, int *volref2)
{
*nvertex = tmpNbVertices[prim];

for(register int vertex = 0; vertex < (*nvertex); ++vertex)
	{
	xvert[vertex] = LengthScale * tmpXvert[prim][vertex];
	yvert[vertex] = LengthScale * tmpYvert[prim][vertex];
	zvert[vertex] = LengthScale * tmpZvert[prim][vertex];
	}
*xnorm = tmpXNorm[prim]; *ynorm = tmpYNorm[prim]; *znorm = tmpZNorm[prim];
*volref1 = tmpVolRef1[prim]; *volref2 = tmpVolRef2[prim];

return 0;
}	// neBEMGetPrimitive ends here


int neBEMVolumeDescription(int volref, int *shape,
                           int *material, double *epsilon,
                           double *potential, double *charge,
                           int *boundarytype)
{
*shape = tmpVolShape[volref];
*material = tmpVolMaterial[volref];
*epsilon = tmpVolEpsilon[volref];
*potential = tmpVolPotential[volref];
*charge = tmpVolCharge[volref];
*boundarytype = tmpVolBoundaryType[volref];

return 0;
}	// neBEMVolumeDescription ends here


int neBEMVolumePoint(double x, double y, double z)
{
return(0);	// dummy routine
}	// neBEMVolumePoint ends here


// Reads in periodicity information.
// ix: PeriodicTypeX (1 - simple, 2 - mirror, 3 - axial, 4 - rotation)
// jx: PeriodicInX (Number of copies internal to neBEM)
// sx: XPeriod
int neBEMGetPeriodicities(int prim, int *ix, int *jx, double *sx,
                                    int *iy, int *jy, double *sy,
                                    int *iz, int *jz, double *sz)
{
*ix = tmpPeriodicTypeX[prim];
*jx = tmpPeriodicInX[prim]; *sx = tmpXPeriod[prim] * LengthScale;
*iy = tmpPeriodicTypeY[prim];
*jy = tmpPeriodicInY[prim]; *sy = tmpYPeriod[prim] * LengthScale;
*iz = tmpPeriodicTypeZ[prim];
*jz = tmpPeriodicInZ[prim]; *sz = tmpZPeriod[prim] * LengthScale;

return 0;
} // neBEMGetPeriodicities end


// Information on bounding planes
// ixmin=0: lower x-plane does not exist
// ixmin=1: lower x-plane does exist
// cxmin: coordinate of lower x-plane
// vxmin: potential of lower x-plane
// Similar for ixmax, iymin, iymax, izmin, izmax
int neBEMGetBoundingPlanes(int *ixmin, double *cxmin, double *vxmin,
                            int *ixmax, double *cxmax, double *vxmax,
                            int *iymin, double *cymin, double *vymin,
                            int *iymax, double *cymax, double *vymax,
                            int *izmin, double *czmin, double *vzmin,
                            int *izmax, double *czmax, double *vzmax)
{
*ixmin = 0; *ixmax = 0;
*iymin = 0; *iymax = 0;
*izmin = 0; *izmax = 0;
*cxmin = 0.0; *cxmax = 0.0;
*cymin = 0.0; *cymax = 0.0;
*czmin = 0.0; *czmax = 0.0;
*vxmin = 0.0; *vxmax = 0.0;
*vymin = 0.0; *vymax = 0.0;
*vzmin = 0.0; *vzmax = 0.0;

return 0;
} // neBEMGetBoundingPlanes ends


int neBEMGetMirror(int prim,
        int* ix, int* jx, double* sx,
        int* iy, int* jy, double* sy,
        int* iz, int* jz, double* sz)
{
*ix = *iy = *iz = 0;  // no mirror now
*jx = *jy = *jz = 0;    // not used at present
*sx = *sy = *sz = 0.0;  // mirror assumed to be passing through the origin

// Only value 2 implies mirror periodicity
// At present MirrorTypeX etc is always 2. When Mirrors of other kinds will be
// allowed, this will need modification.
if((*ix != 2) && (*iy != 2) && (*iz != 2))
  {
  *ix = *iy = *iz = 0;
  }

// Only one reflection is allowed at present
if(*ix == 2)
  { *iy = *iz = 0;}
else if (*iy == 2)
  *iz = 0;

// Return error status.
return 0;
} // end of neBEMGetMirror
