/* solves [A]{X} = {B} for a vector {X}, where [A] is specified by the
   arrays 'u[1...m][1...n]', 'w[1...n]', 'v[1...n][1...n]' as returned by
   'svdcmp'. 'm' and 'n' are the dimensions of [A], and will be equal for
   square matrices. 'b[1...m]' is the input right-hand side. x[1...n] is the
   output solution vector. No input quantities are destroyed, so the
   routine may be called sequentilally with different 'b's. */

#include <NR.h>

void svbksb(double **u, double *w, double **v, int m, int n, double *b, double *x)
{
int		i, j, jj;
double   s, *tmp;

tmp = dvector(1, n);
for(j = 1; j <= n; j++)	/* calculate (transpose)u b */
	{
	s = 0.0;
	if(w[j])			/* nonzero result only if w[j] is nonzero */
		{
		#pragma omp parallel for private(i) reduction(+:s)
		for(i = 1; i <= m; i++)
			s+= u[i][j] * b[i];
		s /= w[j];
		}

	tmp[j] = s;
	}

for(j = 1; j <= n; j++)	/* matrix multiply by [V] to get answer */
	{
	s = 0.0;
	#pragma omp parallel for private(jj) reduction(+:s)
	for(jj = 1; jj <= n; jj++)
		s += v[j][jj] * tmp[jj];
	x[j] = s;
	}

free_dvector(tmp, 1, n);
}
