// PostProcess an Example Device
#include <stdio.h>
#include <assert.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>


// Carry out necessary post-processing depending on MapFile data
int PostProcess(void)
{
int NbMaps, NbLines, NbPoints;
char instr[256];

// Create maps and evaluate necessary properties on these maps
FILE *fMapIn;
fMapIn = fopen(MapFile, "r");
assert(fMapIn != NULL);

fgets(instr, 256, fMapIn); fscanf(fMapIn, "%d\n", &NbMaps);	// number of Maps
if(0) printf("#%s %d\n", instr, NbMaps);

if(NbMaps)	// if NbMaps non-zero
	{
	int NX, NY, NZ, Nblock;
	double LX, LY, LZ, MidX, MidY, MidZ;

	for(register int map=1; map<=NbMaps; ++map)
		{
		fgets(instr, 256, fMapIn); fscanf(fMapIn, "%d %d %d\n", &NX, &NY, &NZ);
		if(0) printf("#%s %d %d %d\n", instr, NX, NY, NZ);

		fgets(instr, 256, fMapIn); fscanf(fMapIn, "%le %le %le\n", &LX, &LY, &LZ);
		if(0) printf("#%s %le %le %le\n", instr, LX, LY, LZ);
		LX*=LengthScale; LY*=LengthScale; LZ*=LengthScale;

		fgets(instr, 256, fMapIn);
		fscanf(fMapIn, "%le %le %le\n", &MidX, &MidY, &MidZ);
		if(0) printf("#%s %le %le %le\n", instr, MidX, MidY, MidZ);
		MidX*=LengthScale; MidY*=LengthScale; MidZ*=LengthScale;

		fgets(instr, 256, fMapIn); fscanf(fMapIn, "%d\n", &Nblock);
		if(0) printf("#%s %d\n", instr, Nblock);

		int size = NX*NY*NZ;
		printf("Number of points on map %d: %d\n", map, size);
		printf("Number of points in each block of the map: %d\n", Nblock);

		if(size)	// if there are points on the map
			{
			double delX, delY, delZ, startX, startY, startZ;
			startX = MidX - LX/2.0; startY = MidY - LY/2.0; startZ = MidZ - LZ/2.0;
			if(NX > 1) delX = LX / (NX-1);
			else delX = 0.0;
			if(NY > 1) delY = LY / (NY-1);
			else delY = 0.0;
			if(NZ > 1) delZ = LZ / (NZ-1);
			else delZ = 0.0;

			char strMap[10], MapOut[256];
			sprintf(strMap, "%d", map);
			strcpy(MapOut, PPOutDir); strcat(MapOut, "/Map");
			strcat(MapOut, strMap); strcat(MapOut, ".out");
			FILE *fMapOut = fopen(MapOut, "w");
			assert(fMapOut != NULL);
			fprintf(fMapOut, "#point.X\tpoint.Y\tpoint.Z\tpot\txfl\tyfl\tzfl\n");

			int np=0, fstatus;
			Point3D point;
			double potential;
			Vector3D field;

			printf("Map point: ");
			for(register int i = 1; i <= NX; ++i)
  			{
  			point.X = startX + (i-1)*delX;
  			for(register int j = 1; j <= NY; ++j)
    			{
    			point.Y = startY + (j-1)*delY;
    			for(register int k = 1; k <= NZ; ++k)
      			{
      			point.Z = startZ + (k-1)*delZ;

						++np;
						printf("%12d", np);
						fstatus = neBEMField(&point, &potential, &field);
						assert(fstatus == 0);

      			fprintf(fMapOut,
												"%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\n",
												point.X/LengthScale, point.Y/LengthScale, point.Z/LengthScale,
												potential/LengthScale,
												field.X, field.Y, field.Z);

						if( (np >= Nblock) && !(np%Nblock))
							fprintf(fMapOut, "\n");
						printf("\b\b\b\b\b\b\b\b\b\b\b\b");
      			}
    			}
  			}
			fclose(fMapOut);
			}	// if size
		}	// for maps
	}	// if NbMaps

fgets(instr, 256, fMapIn); fscanf(fMapIn, "%d\n", &NbLines);	// Nb of Lines
if(0) printf("#%s %d\n", instr, NbLines);

if(NbLines)	// if NbLines non-zero
	{
	int NbSegments;
	double x0, y0, z0, x1, y1, z1;

	for(register int line=1; line<=NbLines; ++line)
		{
		fgets(instr, 256, fMapIn); fscanf(fMapIn, "%le %le %le\n", &x0, &y0, &z0);
		if(0) printf("#%s %lg %lg %lg\n", instr, x0, y0, z0);
		x0 *= LengthScale; y0 *= LengthScale; z0 *= LengthScale;
		fgets(instr, 256, fMapIn); fscanf(fMapIn, "%le %le %le\n", &x1, &y1, &z1);
		if(0) printf("#%s %lg %lg %lg\n", instr, x1, y1, z1);
		x1 *= LengthScale; y1 *= LengthScale; z1 *= LengthScale;
		fgets(instr, 256, fMapIn); fscanf(fMapIn, "%d\n", &NbSegments);
		if(0) printf("#%s %d\n", instr, NbSegments);

		printf("Number of segments on line %d: %d\n", line, NbSegments);

		if(NbSegments)	// if there are non-zero segments on the line
			{
			double delx, dely, delz;

			delx = (x1-x0) / NbSegments;
			dely = (y1-y0) / NbSegments;
			delz = (z1-z0) / NbSegments;

			char strLine[10], LineOut[256];
			sprintf(strLine, "%d", line);
			strcpy(LineOut, PPOutDir); strcat(LineOut, "/Line");
			strcat(LineOut, strLine); strcat(LineOut, ".out");
			FILE *fLineOut = fopen(LineOut, "w");
			assert(fLineOut != NULL);
			fprintf(fLineOut, "#point.X\tpoint.Y\tpoint.Z\tpot\txfl\tyfl\tzfl\n");

			int fstatus;
			Point3D point;
			double potential;
			Vector3D field;

			printf("Line point: ");
			for(register int seg = 1; seg <= NbSegments+1; ++seg)
  			{
				printf("%12d", seg);

  			point.X = x0 + (seg-1)*delx;
   			point.Y = y0 + (seg-1)*dely;
   			point.Z = z0 + (seg-1)*delz;

				fstatus = neBEMField(&point, &potential, &field);
				assert(fstatus == 0);

   			fprintf(fLineOut,
										"%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\n",
											point.X/LengthScale, point.Y/LengthScale,
											point.Z/LengthScale,
											potential/LengthScale,
											field.X, field.Y, field.Z);

				printf("\b\b\b\b\b\b\b\b\b\b\b\b");
  			}	// for seg
			fclose(fLineOut);
			}	// if NbSegments
		}	// for lines
	}	// if NbLines

fgets(instr, 256, fMapIn); fscanf(fMapIn, "%d\n", &NbPoints);	// Nb of Lines
if(0) printf("#%s %d\n", instr, NbPoints);

if(NbPoints > 0)
  {
	char PointOut[256];
	strcpy(PointOut, PPOutDir); strcat(PointOut, "/Points.out");
	FILE *fPointOut = fopen(PointOut, "w");
	assert(fPointOut != NULL);
	fprintf(fPointOut, "#point.X\tpoint.Y\tpoint.Z\tpot\txfl\tyfl\tzfl\n");

	Point3D point;
	double potential;
	Vector3D field;
	int fstatus;

	printf("\n"); printf("Extra point: ");
  for(register int i = 1; i <= NbPoints; ++i)
    {
		printf("%12d", i);
    fscanf(fMapIn, "%le %le %le\n", &point.X, &point.Y, &point.Z);
		point.X *= LengthScale; point.Y *= LengthScale; point.Y *= LengthScale;
		fstatus = neBEMField(&point, &potential, &field);
		assert(fstatus == 0);

   	fprintf(fPointOut, "%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\n",
												point.X/LengthScale, point.Y/LengthScale,
												point.Z/LengthScale,
								potential/LengthScale,
								field.X, field.Y, field.Z);
		printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
    }
	fclose(fPointOut);
  }	// if NbPoints

return 0;
}	// PostProcess
