// Post-Process conducting plate

#include <stdio.h>
#include <assert.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>


// Finds the sum of charges on all the elements
int PostProcess(void)
{
int prim, rptCnt;
double SumCharge = 0.0;

for(register unsigned int elem = 1; elem <= NbElements; ++elem)
	{
	prim = (EleArr+elem-1)->PrimitiveNb; rptCnt = 0;
	if(PeriodicInX[prim] || PeriodicInY[prim] || PeriodicInZ[prim])
		{
		for(register int xrpt = -PeriodicInX[prim];
					xrpt <= PeriodicInX[prim]; ++xrpt)
			for(register int yrpt = -PeriodicInY[prim];
						yrpt <= PeriodicInY[prim]; ++yrpt)
				for(register int zrpt = -PeriodicInZ[prim];
							zrpt <= PeriodicInZ[prim]; ++zrpt)
					if( (xrpt == 0) && (yrpt == 0) && (zrpt == 0) )
						continue;
					else
						++rptCnt;
			
		}	// if Periodic
	else
		rptCnt += 1;

	SumCharge += rptCnt * (EleArr+elem-1)->Solution * (EleArr+elem-1)->G.dA;
	}

printf("Plate capacitance / (4 pi eps0): %lg\n", SumCharge / MyFACTOR);

// A typical evaluation of potential and field is demonstrated below.
/*
Point3D point;
double potential;
Vector3D field;
point.X = 0.0; point.Y = 0.0; point.Z = 0.0;
int fstatus = neBEMField(&point, &potential, &field);
assert(fstatus == 0);

// Find field on the right edge of the base device
char OutputFile[256];
FILE *fout;

strcpy(OutputFile, PPOutDir);
strcat(OutputFile, "/FieldOnRtEdge.out");
fout = fopen(OutputFile, "w");
for(register int iy = -10; iy <= 10; ++iy)
	{
	point.Y = (double)iy * 0.01;
	for(register int iz = -10; iz <= 10; ++iz)
		{
		printf("iy: %4d, iz: %4d", iy, iz);
		point.Z = (double)iz * 0.01;
		PFAtPoint(&point, &potential, &field);
		fprintf(fout, "%lg %lg %lg %lg %lg %lg %lg\n",
										point.X, point.Y, point.Z, field.X, field.Y, field.Z,
										sqrt(field.X*field.X + field.Y*field.Y + field.Z*field.Z));
		printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
		}
	fprintf(fout, "\n");
	}

fclose(fout);
*/

return 0;
}
