#include <stdio.h>
#include <assert.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>
#include <Isles.h>


// This part is for post-processing and can be as simple and as elaborate
// as the user decides. A typical evaluation of potential and field is
// demonstrated below.

// Finds potential and field for comparison with known numerical solution
int PostProcess(void)
{
int fstatus;
Point3D point;
double potential;
Vector3D field;

point.X = 0.0; point.Y = 0.0; point.Z = 0.0;
fstatus = neBEMField(&point, &potential, &field);
assert(fstatus == 0);
printf("point.X: %lg, point.Y: %lg, point.Z: %lg\n", point.X, point.Y, point.Z);
printf("Potential: %lg\n", potential);
printf("field.X: %lg, field.Y: %lg, field.Z: %lg\n", field.X, field.Y, field.Z);

point.X = 0.0; point.Y = 5.0e-2; point.Z = 0.0;
fstatus = neBEMField(&point, &potential, &field);
assert(fstatus == 0);
printf("point.X: %lg, point.Y: %lg, point.Z: %lg\n", point.X, point.Y, point.Z);
printf("Potential: %lg\n", potential);
printf("field.X: %lg, field.Y: %lg, field.Z: %lg\n", field.X, field.Y, field.Z);

point.X = 0.0; point.Y = -5.0e-2; point.Z = 0.0;
fstatus = neBEMField(&point, &potential, &field);
assert(fstatus == 0);
printf("point.X: %lg, point.Y: %lg, point.Z: %lg\n", point.X, point.Y, point.Z);
printf("Potential: %lg\n", potential);
printf("field.X: %lg, field.Y: %lg, field.Z: %lg\n", field.X, field.Y, field.Z);

point.X = 5.0e-2; point.Y = 5.0e-2; point.Z = 5.0e-2;
fstatus = neBEMField(&point, &potential, &field);
assert(fstatus == 0);
printf("point.X: %lg, point.Y: %lg, point.Z: %lg\n", point.X, point.Y, point.Z);
printf("Potential: %lg\n", potential);
printf("field.X: %lg, field.Y: %lg, field.Z: %lg\n", field.X, field.Y, field.Z);

int NbMaps=3;

if(NbMaps)  // if NbMaps non-zero
  {
  int NX, NY, NZ, Nblock;
  double LX, LY, LZ, MidX, MidY, MidZ;

  for(register int map=1; map<=NbMaps; ++map)
    {
		if(map == 1)
			{
			NX = 101; NY = 101; NZ = 1;
			LX = 3.0e-2; LY = 3.0e-2; LZ = 0.0;
			Nblock = NY;
			}
		else if(map == 2)
			{
			NX = 1; NY = 101; NZ = 101;
			LX = 0.0e-2; LY = 3.0e-2; LZ = 3.0e-2;
			Nblock = NZ;
			}
		else if(map == 3)
			{
			NX = 101; NY = 1; NZ = 101;
			LX = 3.0e-2; LY = 0.0e-2; LZ = 3.0e-2;
			Nblock = NZ;
			}

    LX*=LengthScale; LY*=LengthScale; LZ*=LengthScale;

		MidX = 0.0; MidY = 0.5e-2; MidZ = 0.0;
    MidX*=LengthScale; MidY*=LengthScale; MidZ*=LengthScale;

    int size = NX*NY*NZ;
    printf("Number of points on map %d: %d\n", map, size);
    printf("Number of points in each block of the map: %d\n", Nblock);

    if(size)  // if there are points on the map
      {
      double delX, delY, delZ, startX, startY, startZ;
      startX = MidX - LX/2.0; startY = MidY - LY/2.0; startZ = MidZ - LZ/2.0;
      if(NX > 1) delX = LX / (NX-1);
      else delX = 0.0;
      if(NY > 1) delY = LY / (NY-1);
      else delY = 0.0;
      if(NZ > 1) delZ = LZ / (NZ-1);
      else delZ = 0.0;

      char strMap[10], MapOut[256];
      sprintf(strMap, "%d", map);
      strcpy(MapOut, PPOutDir); strcat(MapOut, "/Map");
      strcat(MapOut, strMap); strcat(MapOut, ".out");
      FILE *fMapOut = fopen(MapOut, "w");
      assert(fMapOut != NULL);
      fprintf(fMapOut, "#point.X\tpoint.Y\tpoint.Z\tpot\txfl\tyfl\tzfl\n");

      int np=0, fstatus;
      Point3D point;
      double potential;
      Vector3D field;

      printf("Map point: ");
      for(register int i = 1; i <= NX; ++i)
        {
        point.X = startX + (i-1)*delX;
        for(register int j = 1; j <= NY; ++j)
          {
          point.Y = startY + (j-1)*delY;
          for(register int k = 1; k <= NZ; ++k)
            {
            point.Z = startZ + (k-1)*delZ;

            ++np;
            printf("%12d", np);
            fstatus = neBEMField(&point, &potential, &field);
            assert(fstatus == 0);

            fprintf(fMapOut,
                        "%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\n",
                        point.X/LengthScale, point.Y/LengthScale,
												point.Z/LengthScale,
                        potential/LengthScale,
                        field.X, field.Y, field.Z);

            if( (np >= Nblock) && !(np%Nblock))
              fprintf(fMapOut, "\n");
            printf("\b\b\b\b\b\b\b\b\b\b\b\b");
            }	// k
          }	// j
        }	// i
      fclose(fMapOut);
      } // if size
    } // for maps
  } // if NbMaps

int NbLines = 3;

if(NbLines) // if NbLines non-zero
  {
  int NbSegments;
  double x0, y0, z0, x1, y1, z1;

  for(register int line=1; line<=NbLines; ++line)
    {
		if(line == 1)
			{
			x0 = -10.0e-2; y0 = 0.0; z0 = 0.0;
			x1 = +10.0e-2; y1 = 0.0; z1 = 0.0;
			}
		else if(line == 2)
			{
			x0 = 0.0e-2; y0 = -10.0e-2; z0 = 0.0;
			x1 = 0.0e-2; y1 = +10.0e-2; z1 = 0.0;
			}
		else if(line == 3)
			{
			x0 = 0.0e-2; y0 = 0.0; z0 = -10.0e-2;
			x1 = 0.0e-2; y1 = 0.0; z1 = +10.0e-2;
			}

    x0 *= LengthScale; y0 *= LengthScale; z0 *= LengthScale;
    x1 *= LengthScale; y1 *= LengthScale; z1 *= LengthScale;

		NbSegments = 1001;

    printf("Number of segments on line %d: %d\n", line, NbSegments);

    if(NbSegments)  // if there are non-zero segments on the line
      {
      double delx, dely, delz;

      delx = (x1-x0) / NbSegments;
      dely = (y1-y0) / NbSegments;
      delz = (z1-z0) / NbSegments;

      char strLine[10], LineOut[256];
      sprintf(strLine, "%d", line);
      strcpy(LineOut, PPOutDir); strcat(LineOut, "/Line");
      strcat(LineOut, strLine); strcat(LineOut, ".out");
      FILE *fLineOut = fopen(LineOut, "w");
      assert(fLineOut != NULL);
      fprintf(fLineOut, "#point.X\tpoint.Y\tpoint.Z\tpot\txfl\tyfl\tzfl\n");

      int fstatus;
      Point3D point;
      double potential;
      Vector3D field;

      printf("Line point: ");
      for(register int seg = 1; seg <= NbSegments+1; ++seg)
        {
        printf("%12d", seg);

        point.X = x0 + (seg-1)*delx;
        point.Y = y0 + (seg-1)*dely;
        point.Z = z0 + (seg-1)*delz;

        fstatus = neBEMField(&point, &potential, &field);
        assert(fstatus == 0);

        fprintf(fLineOut,
                    "%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\n",
                      point.X/LengthScale, point.Y/LengthScale,
											point.Z/LengthScale,
                      potential/LengthScale,
                      field.X, field.Y, field.Z);

        printf("\b\b\b\b\b\b\b\b\b\b\b\b");
        } // for seg
      fclose(fLineOut);
      } // if NbSegments
    } // for lines
  } // if NbLines

int NbPoints = 0;

if(NbPoints > 0)
  {
  char PointOut[256];
  strcpy(PointOut, PPOutDir); strcat(PointOut, "/Points.out");
  FILE *fPointOut = fopen(PointOut, "w");
  assert(fPointOut != NULL);
  fprintf(fPointOut, "#point.X\tpoint.Y\tpoint.Z\tpot\txfl\tyfl\tzfl\n");

  Point3D point;
  double potential;
  Vector3D field;
  int fstatus;
  printf("\n"); printf("Extra point: ");
  for(register int i = 1; i <= NbPoints; ++i)
    {
    printf("%12d", i);
		point.X = 0.0; point.Y = 0.0; point.Z = 0.0;
    point.X *= LengthScale; point.Y *= LengthScale; point.Y *= LengthScale;
    fstatus = neBEMField(&point, &potential, &field);
    assert(fstatus == 0);

    fprintf(fPointOut, "%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\t%.8lg\n",
                				point.X/LengthScale, point.Y/LengthScale,
												point.Z/LengthScale,
                potential/LengthScale,
                field.X, field.Y, field.Z);
    printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
    }
  fclose(fPointOut);
  } // if NbPoints

return 0;
}
