/*
(c) 2005, Supratik Mukhopadhayay, Nayana Majumdar
*/
// Generates the coordinates of the locations at which potential and
// electric fields will be computed by neBEM V7
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char *argv[])
{
int NX, NY, NZ,
	size, NLblock, NExtra;
double LX, LY, LZ, MidX, MidY, MidZ,
	   delX, delY, delZ, startX, startY, startZ,
	   xpos, ypos, zpos;
char instr[256];
FILE *fin, *fp, *fe;

if(argc != 3)
	{
	printf("Wrong number of arguments used for fieldmaps.\n");
	printf("Usage:\n");
	printf("fieldmaps <PMap.inp> <FMap.inp>\n");
	printf("\t\twhere PMap.inp and FMap.inp are names of input files.\n");
	printf("Exiting ... better luck next time.\n");
	exit(-2);
	}
fin = fopen(argv[1], "r");
if(fin == NULL) { printf("No %s! ... exiting ...\n", argv[1]); exit(-1); }

fgets(instr, 256, fin); fscanf(fin, "%d\n", &NX);
// printf("#%s %d\n", instr, NX);
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NY);
// printf("#%s %d\n", instr, NY);
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NZ);
// printf("#%s %d\n", instr, NZ);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &LX);
// printf("#%s %le\n", instr, LX);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &LY);
// printf("#%s %le\n", instr, LY);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &LZ);
// printf("#%s %le\n", instr, LZ);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &MidX);
// printf("#%s %le\n", instr, MidX);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &MidY);
// printf("#%s %le\n", instr, MidY);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &MidZ);
// printf("#%s %le\n", instr, MidZ);
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NLblock);
// printf("#%s %d\n", instr, NLblock);
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NExtra);
// printf("#%s %d\n", instr, NExtra);
fgets(instr, 256, fin);
// printf("#%s\n", instr);

// fscanf(fin, "%d %d %d\n", &NX, &NY, &NZ);
// fscanf(fin, "%lf %lf %lf\n", &LX, &LY, &LZ);
// fscanf(fin, "%lf %lf %lf\n", &MidX, &MidY, &MidZ);
// fscanf(fin, "%d", &NLblock);
// fscanf(fin, "%d", &NExtra);

size = NX*NY*NZ;
fp = fopen("Data/PField.map", "w");
if(fp == NULL)
	{
	printf("Cannot open Data/PField.map for writing ... exiting.\n");
	exit(-1);
	}
fprintf(fp, "%d %d %d\n", size, NLblock, NExtra);

startX = MidX - LX/2.0; startY = MidY - LY/2.0; startZ = MidZ - LZ/2.0;
if(NX > 1)
	delX = LX / (NX-1);
else
	delX = 0.0;
if(NY > 1)
	delY = LY / (NY-1);
else
	delY = 0.0;
if(NZ > 1)
	delZ = LZ / (NZ-1);
else
	delZ = 0.0;

for(register int i = 1; i <= NX; ++i)
	{
	xpos = startX + (i-1)*delX;
	for(register int j = 1; j <= NY; ++j)
		{
		ypos = startY + (j-1)*delY;
		for(register int k = 1; k <= NZ; ++k)
			{
			zpos = startZ + (k-1)*delZ;

			fprintf(fp, "%le\t%le\t%le\n", xpos, ypos, zpos);	
			}
		}
	}

if(NExtra > 0)
	{
	for(register int i = 1; i <= NExtra; ++i)
		{
		fscanf(fin, "%le %le %le\n", &xpos, &ypos, &zpos);
		fprintf(fp, "%le\t%le\t%le\n", xpos, ypos, zpos);	
		}
	}

fclose(fin);
fclose(fp);

fin = fopen(argv[2], "r");
if(fin == NULL) { printf("No %s! ... exiting ...\n", argv[2]); exit(-1); }

fgets(instr, 256, fin); fscanf(fin, "%d\n", &NX);
// printf("#%s %d\n", instr, NX);
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NY);
// printf("#%s %d\n", instr, NY);
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NZ);
// printf("#%s %d\n", instr, NZ);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &LX);
// printf("#%s %le\n", instr, LX);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &LY);
// printf("#%s %le\n", instr, LY);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &LZ);
// printf("#%s %le\n", instr, LZ);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &MidX);
// printf("#%s %le\n", instr, MidX);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &MidY);
// printf("#%s %le\n", instr, MidY);
fgets(instr, 256, fin); fscanf(fin, "%le\n", &MidZ);
// printf("#%s %le\n", instr, MidZ);
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NLblock);
// printf("#%s %d\n", instr, NLblock);
fgets(instr, 256, fin); fscanf(fin, "%d\n", &NExtra);
// printf("#%s %d\n", instr, NExtra);
fgets(instr, 256, fin);
// printf("#%s\n", instr);

// fscanf(fin, "%d %d %d\n", &NX, &NY, &NZ);
// fscanf(fin, "%lf %lf %lf\n", &LX, &LY, &LZ);
// fscanf(fin, "%lf %lf %lf\n", &MidX, &MidY, &MidZ);
// fscanf(fin, "%d", &NLblock);
// fscanf(fin, "%d", &NExtra);

size = NX*NY*NZ;
fe = fopen("Data/FField.map", "w");
if(fe == NULL)
	{
	printf("Cannot open Data/PField.map for writing ... exiting.\n");
	exit(-1);
	}
fprintf(fe, "%d %d %d\n", size, NLblock, NExtra);

startX = MidX - LX/2.0; startY = MidY - LY/2.0; startZ = MidZ - LZ/2.0;
if(NX > 1)
    delX = LX / (NX-1);
else
    delX = 0.0;
if(NY > 1)
    delY = LY / (NY-1);
else
    delY = 0.0;
if(NZ > 1)
    delZ = LZ / (NZ-1);
else
    delZ = 0.0;

for(register int i = 1; i <= NX; ++i)
    {
    xpos = startX + (i-1)*delX;
    for(register int j = 1; j <= NY; ++j)
        {
        ypos = startY + (j-1)*delY;
        for(register int k = 1; k <= NZ; ++k)
            {
            zpos = startZ + (k-1)*delZ;

            fprintf(fe, "%le\t%le\t%le\n", xpos, ypos, zpos);
            }
        }
    }

if(NExtra > 0)
	{
	for(register int i = 1; i <= NExtra; ++i)
		{
		fscanf(fin, "%le %le %le\n", &xpos, &ypos, &zpos);
		fprintf(fe, "%le\t%le\t%le\n", xpos, ypos, zpos);	
		}
	}

fclose(fin);
fclose(fe);
}
